@@ -5,11 +5,16 @@ import os, string
 from AccessControl import ClassSecurityInfo
 import Globals
 from OFS.ObjectManager import bad_id
+from StructuredText.StructuredText import HTML
 from ZPublisher.Publish import call_object, missing_name, dont_publish_class
 from ZPublisher.mapply import mapply
 from Products.CMFFormController import GLOBALS as fc_globals
-from Products.CMFCore.utils import getToolByName, UniqueObject, SimpleItemWithProperties, format_stx
-from Products.CMFCore.CMFCorePermissions import ManagePortal
+from Products.CMFCore.utils import getToolByName, UniqueObject, SimpleItemWithProperties
+# BBB CMF < 1.5
+try:
+    from Products.CMFCore.permissions import ManagePortal
+except ImportError:
+    from Products.CMFCore.CMFCorePermissions import ManagePortal
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from Products.CMFFormController.ControllerState import ControllerState
 from FormAction import FormActionType, FormActionKey, FormAction, FormActionContainer
@@ -71,7 +76,7 @@ class FormController(UniqueObject, SimpleItemWithProperties):
     f = open(os.path.join(wwwpath, 'docs.stx'), 'r')
     _docs = f.read()
     f.close()
-    _docs = format_stx(_docs)
+    _docs = HTML(_docs)
 
 
     def __init__(self):
@@ -351,8 +356,10 @@ class FormController(UniqueObject, SimpleItemWithProperties):
                 kwargs = argdict
             else:
                 kwargs = {}
-                for k, v in REQUEST.items():
-                    kwargs[k] = v
+                for k in REQUEST.keys():
+                    if k in ('SESSION',):
+                        continue
+                    kwargs[k] = REQUEST[k]
                 kwargs.update(argdict)
         context = controller_state.getContext()
         if validators is None:
@@ -407,8 +414,16 @@ class FormController(UniqueObject, SimpleItemWithProperties):
         return type_name
 
 
-    security.declareProtected(ManagePortal, 'purge')
-    def purge(self):
+    security.declareProtected(ManagePortal, 'manage_purge')
+    def manage_purge(self, REQUEST):
+        """TTW interface for purge"""
+        text = self._purge()
+        # might want to do a redirect here instead
+        return text
+
+
+    security.declareProtected(ManagePortal, '_purge')
+    def _purge(self):
         """Remove actions and validators for ids that no longer correspond to
         objects in the portal"""
         actions = self.actions.getFiltered()
