@@ -1,21 +1,24 @@
-import os, sys
-import urllib
+import os
 import Globals
-from DateTime import DateTime
 from AccessControl import Owned, ClassSecurityInfo, getSecurityManager
-from Acquisition import aq_parent, aq_base, aq_inner, aq_chain, aq_get
-from OFS.ObjectManager import ObjectManager
+from AccessControl.Permission import Permission
+from Acquisition import aq_parent, aq_base, aq_inner, aq_get
 from OFS.SimpleItem import SimpleItem
+from StructuredText.StructuredText import HTML
 from ZPublisher.Publish import call_object, missing_name, dont_publish_class
 from ZPublisher.mapply import mapply
 from Products.CMFPlone import cmfplone_globals
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
-from Products.CMFCore import CMFCorePermissions
-from Products.CMFCore.utils import UniqueObject, getToolByName, format_stx
-from Products.CMFCore.Skinnable import SkinnableObjectManager
+from Products.CMFCore.permissions import ManagePortal
+from Products.CMFCore.utils import UniqueObject, getToolByName
 from Products.CMFPlone.PloneFolder import PloneFolder as TempFolderBase
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
+from Products.CMFPlone.utils import base_hasattr
+from ZODB.POSException import ConflictError
 
+ListType=type([])
+
+FACTORY_INFO = '__factory__info__'
 
 # ##############################################################################
 # A class used for generating the temporary folder that will
@@ -27,8 +30,6 @@ class TempFolder(TempFolderBase):
     portal_type = meta_type = 'TempFolder'
     isPrincipiaFolderish = 0
 
-    parent = None
-
     # override getPhysicalPath so that temporary objects return a full path
     # that includes the acquisition parent of portal_factory (otherwise we get
     # portal_root/portal_factory/... no matter where the object will reside)
@@ -39,8 +40,8 @@ class TempFolder(TempFolderBase):
         and getPhysicalPath() are designed to operate together.
         '''
         portal_factory = aq_parent(aq_inner(self))
-        path = aq_parent(portal_factory).getPhysicalPath() + (portal_factory.getId(), self.getId(),)
-
+        path = aq_parent(portal_factory).getPhysicalPath() + \
+            (portal_factory.getId(), self.getId(),)
         return path
 
     # override / delegate local roles methods
@@ -49,8 +50,7 @@ class TempFolder(TempFolderBase):
         Zope's and GRUF's User.getRolesInContext both walk up the
         acquisition hierarchy using aq_parent(aq_inner(obj)) when
         they gather local roles, and this process will result in
-        their walking from TempFolder to portal_factory to the portal root.
-        XXX If we implement PLIP16, this will have to be modified!!!"""
+        their walking from TempFolder to portal_factory to the portal root."""
         object = aq_parent(aq_parent(self))
         local_roles = {}
         while 1:
@@ -67,6 +67,11 @@ class TempFolder(TempFolderBase):
                         if not role in local_roles[k]:
                             local_roles[k].append(role)
 
+            # Check if local role has to be acquired (PLIP 16)
+            if getattr(object, '__ac_local_roles_block__', None):
+                # Ok, we have to stop there, as lr. blocking is enabled
+                break
+
             # Prepare next iteration
             inner = getattr(object, 'aq_inner', object)
             parent = getattr(inner, 'aq_parent', None)
@@ -112,13 +117,10 @@ class TempFolder(TempFolderBase):
     def userCanTakeOwnership(self):
         return aq_parent(aq_parent(self)).userCanTakeOwnership()
 
-
     # delegate allowedContentTypes
     def allowedContentTypes(self):
         return aq_parent(aq_parent(self)).allowedContentTypes()
 
-
-    # override __getitem__
     def __getitem__(self, id):
         # Zope's inner acquisition chain for objects returned by __getitem__ will be
         # portal -> portal_factory -> temporary_folder -> object
@@ -127,6 +129,11 @@ class TempFolder(TempFolderBase):
         # So we need to rewrap...
         portal_factory = aq_parent(self)
         intended_parent = aq_parent(portal_factory)
+
+        # If the intended parent has an object with the given id, just do a passthrough
+        if hasattr(intended_parent, id):
+            return getattr(intended_parent, id)
+
         # rewrap portal_factory
         portal_factory = aq_base(portal_factory).__of__(intended_parent)
         # rewrap self
@@ -136,9 +143,27 @@ class TempFolder(TempFolderBase):
             return (aq_base(self._getOb(id)).__of__(temp_folder)).__of__(intended_parent)
         else:
             type_name = self.getId()
-            self.invokeFactory(id=id, type_name=type_name)
+            try:
+                self.invokeFactory(id=id, type_name=type_name)
+            except ConflictError:
+                raise
+            except:
+                # some errors from invokeFactory (AttributeError, maybe others)
+                # get swallowed -- dump the exception to the log to make sure
+                # developers can see what's going on
+                getToolByName(self, 'plone_utils').logException()
+                raise
             obj = self._getOb(id)
+            
+            # keep obj out of the catalog
             obj.unindexObject()
+
+            # additionally keep it out of Archetypes UID and refs catalogs
+            if base_hasattr(obj, '_uncatalogUID'):
+                obj._uncatalogUID(obj)
+            if base_hasattr(obj, '_uncatalogRefs'):
+                obj._uncatalogRefs(obj)
+
             return (aq_base(obj).__of__(temp_folder)).__of__(intended_parent)
 
     # ignore rename requests since they don't do anything
@@ -146,7 +171,6 @@ class TempFolder(TempFolderBase):
         pass
 
 
-
 # ##############################################################################
 class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
     """ """
@@ -155,7 +179,7 @@ class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
     toolicon = 'skins/plone_images/add_icon.gif'
     security = ClassSecurityInfo()
     isPrincipiaFolderish = 0
-    
+
     __implements__ = (PloneBaseTool.__implements__, SimpleItem.__implements__, )
 
     manage_options = ( ({'label':'Overview', 'action':'manage_overview'}, \
@@ -163,19 +187,19 @@ class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
                         {'label':'Factory Types', 'action':'manage_portal_factory_types'},) +
                        SimpleItem.manage_options)
 
-    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
+    security.declareProtected(ManagePortal, 'manage_overview')
     manage_overview = PageTemplateFile('www/portal_factory_manage_overview', globals())
     manage_overview.__name__ = 'manage_overview'
     manage_overview._need__name__ = 0
 
-    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_portal_factory_types')
+    security.declareProtected(ManagePortal, 'manage_portal_factory_types')
     manage_portal_factory_types = PageTemplateFile(os.path.join('www', 'portal_factory_manage_types'), globals())
     manage_portal_factory_types.__name__ = 'manage_portal_factory_types'
     manage_portal_factory_types._need__name__ = 0
 
     manage_main = manage_overview
 
-    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_docs')
+    security.declareProtected(ManagePortal, 'manage_docs')
     manage_docs = PageTemplateFile(os.path.join('www','portal_factory_manage_docs'), globals())
     manage_docs.__name__ = 'manage_docs'
 
@@ -183,22 +207,21 @@ class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
     f = open(os.path.join(wwwpath, 'portal_factory_docs.stx'), 'r')
     _docs = f.read()
     f.close()
-    _docs = format_stx(_docs)
+    _docs = HTML(_docs)
 
     security.declarePublic('docs')
     def docs(self):
         """Returns FactoryTool docs formatted as HTML"""
         return self._docs
 
-
     def getFactoryTypes(self):
         if not hasattr(self, '_factory_types'):
             self._factory_types = {}
         return self._factory_types
 
-
-    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setPortalFactoryTypes')
+    security.declareProtected(ManagePortal, 'manage_setPortalFactoryTypes')
     def manage_setPortalFactoryTypes(self, REQUEST=None, listOfTypeIds=None):
+        """Set the portal types that should use the factory."""
         if listOfTypeIds is not None:
             dict = {}
             for l in listOfTypeIds:
@@ -216,47 +239,38 @@ class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
         if REQUEST:
             REQUEST.RESPONSE.redirect('manage_main')
 
-
     def doCreate(self, obj, id=None, **kw):
         """Create a real object from a temporary object."""
-        if not self.isTemporary(obj=obj):
-            return obj
-        else:
+        if self.isTemporary(obj=obj):
             if id is not None:
                 id = id.strip()
-            if hasattr(obj, 'getId') and callable(getattr(obj, 'getId')):
-                obj_id = obj.getId()
-            else:
-                obj_id = getattr(id, 'id', None)
-            if obj_id is None:
-                raise Exception  # XXX - FIXME
             if not id:
-                id = obj_id
-            type_name = aq_parent(aq_inner(obj)).id
+                if hasattr(obj, 'getId') and callable(getattr(obj, 'getId')):
+                    id = obj.getId()
+                else:
+                    id = getattr(obj, 'id', None)
+            type_name = aq_parent(aq_inner(obj)).id  # get the ID of the TempFolder
             folder = aq_parent(aq_parent(aq_parent(aq_inner(obj))))
             folder.invokeFactory(id=id, type_name=type_name)
             obj = getattr(folder, id)
 
-            # give ownership to currently authenticated member if not anonymous
+            # give ownership to currently authenticated member if not anonymous  XXX is this necessary?
             membership_tool = getToolByName(self, 'portal_membership')
             if not membership_tool.isAnonymousUser():
                 member = membership_tool.getAuthenticatedMember()
                 obj.changeOwnership(member.getUser(), 1)
-                obj.manage_setLocalRoles(member.getUserName(), ['Owner'])
+            if hasattr(aq_base(obj), 'manage_afterPortalFactoryCreate'):
+                obj.manage_afterPortalFactoryCreate()
+        return obj
 
-            return obj
-
-
-    def fixRequest(self):
+    def _fixRequest(self):
         """Our before_publishing_traverse call mangles URL0.  This fixes up
         the REQUEST."""
-        factory_info = self.REQUEST.get('__factory_info__', {})
-        stack = factory_info.get('stack',[])
-        if stack:
-            URL = self.REQUEST.URL0 + '/' + '/'.join(stack)
-        else:
-            URL = self.REQUEST.URL0
-
+        factory_info = self.REQUEST.get(FACTORY_INFO, None)
+        if not factory_info:
+            return
+        stack = factory_info['stack']
+        URL = self.REQUEST.URL0 + '/' + '/'.join(stack)
         self.REQUEST.set('URL', URL)
 
         url_list = URL.split('/')
@@ -271,120 +285,147 @@ class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
         while m < n:
             self.REQUEST.set('BASE%d' % m, '/'.join(url_list[0:len(url_list)-n+1+m]))
             m = m + 1
-        # XXX fix URLPATHn here too
-
+        # TODO fix URLPATHn, BASEPATHn here too
 
     def isTemporary(self, obj):
         """Check to see if an object is temporary"""
-        ob = aq_parent(aq_inner(obj))
+        ob = aq_base(aq_parent(aq_inner(obj)))
         return hasattr(ob, 'meta_type') and ob.meta_type == TempFolder.meta_type
 
-
     def __before_publishing_traverse__(self, other, REQUEST):
-        # prevent further traversal
-        stack = REQUEST.get('TraversalRequestNameStack')
-        stack.reverse()
-        REQUEST.set('TraversalRequestNameStack', [])
-        factory_info = {'stack':stack}
-        REQUEST.set('__factory_info__', factory_info)
+        if REQUEST.get(FACTORY_INFO, None):
+            del REQUEST[FACTORY_INFO]
 
+        stack = REQUEST.get('TraversalRequestNameStack')
+        stack = [str(s) for s in stack]  # convert from unicode if necessary (happens in Epoz for some weird reason)
+        # need 2 more things on the stack at least for portal_factory to kick in:
+        #    (1) a type, and (2) an id
+        if len(stack) < 2: # ignore
+            return
+        type_name = stack[-1]
+        types_tool = getToolByName(self, 'portal_types')
+        # make sure this is really a type name
+        if not type_name in types_tool.listContentTypes():
+            return # nope -- do nothing
+
+        id = stack[-2]
+        intended_parent = aq_parent(self)
+        if hasattr(intended_parent, id):
+            return # do normal traversal via __bobo_traverse__
+
+        # about to create an object - further traversal will be prevented
+        #
+        # before halting traversal, check for method aliases
+        # stack should be [...optional stuff..., id, type_name]
+        key = stack and stack[-3] or '(Default)'
+        ti = types_tool.getTypeInfo(type_name)
+        method_id = ti and ti.queryMethodID(key)
+        if method_id:
+            if key != '(Default)':
+                del(stack[-3])
+            if method_id != '(Default)':
+                stack.insert(-2, method_id)
+            REQUEST._hacked_path = 1
+        
+        stack.reverse()
+        factory_info = {'stack':stack}
+        REQUEST.set(FACTORY_INFO, factory_info)
+        REQUEST.set('TraversalRequestNameStack', [])
+
+    def __bobo_traverse__(self, REQUEST, name):
+        # __bobo_traverse__ can be invoked directly by a restricted_traverse method call
+        # in which case the traversal stack will not have been cleared by __before_publishing_traverse__
+        name = str(name) # fix unicode weirdness
+        types_tool = getToolByName(self, 'portal_types')
+        if not name in types_tool.listContentTypes():
+            return getattr(self, name) # not a type name -- do the standard thing
+        return self._getTempFolder(str(name)) # a type name -- return a temp folder
 
     security.declarePublic('__call__')
     def __call__(self, *args, **kwargs):
         """call method"""
-
-        factory_info = self.REQUEST.get('__factory_info__', {})
-        if not factory_info.get('fixed_request'):
-            self.fixRequest()
-            factory_info['fixed_request'] = 1
-            self.REQUEST.set('__factory_info__', factory_info)
-
+        self._fixRequest()
+        factory_info = self.REQUEST.get(FACTORY_INFO, {})
         stack = factory_info['stack']
-        # stack.reverse()
-        if len(stack) < 2:
-            obj = self.restrictedTraverse('/'.join(stack))
-            if args == ():
-                # XXX hideous hack -- why isn't REQUEST passed in in args??
-                args = (self.REQUEST, )
-            return mapply(obj, self.REQUEST.args, self.REQUEST,
-                               call_object, 1, missing_name, dont_publish_class,
-                               self.REQUEST, bind=1)
+        type_name = stack[0]
         id = stack[1]
 
+        # do a passthrough if parent contains the id
         if id in aq_parent(self).objectIds():
             return aq_parent(self).restrictedTraverse('/'.join(stack[1:]))(*args, **kwargs)
 
-        tempFolder = self.getTempFolder(stack[0])
-
-        # Get the first item in the stack by explicitly calling __getitem__ 
-        # This fixes some problematic interactions with SpeedPack.
-        temp_obj = tempFolder.__getitem__(stack[1])
+        tempFolder = self._getTempFolder(type_name)
+        # Mysterious hack that fixes some problematic interactions with SpeedPack:
+        #   Get the first item in the stack by explicitly calling __getitem__
+        temp_obj = tempFolder.__getitem__(id)
         stack = stack[2:]
         if stack:
             obj = temp_obj.restrictedTraverse('/'.join(stack))
         else:
             obj = temp_obj
-
         return mapply(obj, self.REQUEST.args, self.REQUEST,
                                call_object, 1, missing_name, dont_publish_class,
                                self.REQUEST, bind=1)
 
-
     index_html = None  # call __call__, not index_html
 
-
-    def getTempFolder(self, type_name):
-        factory_info = self.REQUEST.get('__factory_info__', {})
+    def _getTempFolder(self, type_name):
+        
+        factory_info = self.REQUEST.get(FACTORY_INFO, {})
         tempFolder = factory_info.get(type_name, None)
-        if not tempFolder:
-            type_name = urllib.unquote(type_name)
-            # make sure we can add an object of this type to the temp folder
-            types_tool = getToolByName(self, 'portal_types')
-            if not type_name in types_tool.listContentTypes():
-                raise ValueError, 'Unrecognized type %s\n' % type_name
-            if not type_name in types_tool.TempFolder.allowed_content_types:
-                # update allowed types for tempfolder
-                types_tool.TempFolder.allowed_content_types=(types_tool.listContentTypes())
+        if tempFolder:
+            tempFolder = aq_inner(tempFolder).__of__(self)
+            return tempFolder
+        
+        # make sure we can add an object of this type to the temp folder
+        types_tool = getToolByName(self, 'portal_types')
+        if not type_name in types_tool.TempFolder.allowed_content_types:
+            # update allowed types for tempfolder
+            types_tool.TempFolder.allowed_content_types=(types_tool.listContentTypes())
+            
+        tempFolder = TempFolder(type_name).__of__(self)
+        intended_parent = aq_parent(self)
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        folder_roles = {} # mapping from permission name to list or tuple of roles
+                          # list if perm is acquired; tuple if not
+        n_acquired = 0    # number of permissions that are acquired
+
+        # build initial folder_roles dictionary
+        for p in intended_parent.ac_inherited_permissions(1):
+            name, value = p[:2]
+            p=Permission(name,value,intended_parent)
+            roles = p.getRoles()
+            folder_roles[name] = roles
+            if type(roles) is ListType:
+                n_acquired += 1
+
+        # If intended_parent is not the portal, walk up the acquisition hierarchy and
+        # acquire permissions explicitly so we can assign the acquired version to the
+        # temp_folder.  In addition to being cumbersome, this is undoubtedly very slow.
+        if intended_parent != portal:
+            parent = aq_parent(aq_inner(intended_parent))
+            while(n_acquired and parent!=portal):
+                n_acquired = 0
+                for p in parent.ac_inherited_permissions(1):
+                    name, value = p[:2]
+                    roles = folder_roles[name]
+                    if type(roles) is ListType:
+                        p=Permission(name,value,parent)
+                        aq_roles=p.getRoles()
+                        for r in aq_roles:
+                            if not r in roles:
+                                roles.append(r)
+                        if type(aq_roles) is ListType:
+                            n_acquired += 1
+                        else:
+                            roles = tuple(roles)
+                        folder_roles[name] = roles
+                parent = aq_parent(aq_inner(parent))
+        for name, roles in folder_roles.items():
+            tempFolder.manage_permission(name, roles, acquire=type(roles) is ListType)
 
-            tempFolder = TempFolder(type_name)
-            tempFolder.parent = aq_parent(self)
-            tempFolder = aq_inner(tempFolder).__of__(self)
-            tempFolder.manage_permission(CMFCorePermissions.AddPortalContent, ('Anonymous','Authenticated',), acquire=0 )
-            tempFolder.manage_permission(CMFCorePermissions.ModifyPortalContent, ('Anonymous','Authenticated',), acquire=0 )
-            tempFolder.manage_permission('Copy or Move', ('Anonymous','Authenticated',), acquire=0 )
-        else:
-            tempFolder = aq_inner(tempFolder).__of__(self)
         factory_info[type_name] = tempFolder
-        self.REQUEST.set('__factory_info__', factory_info)
-
+        self.REQUEST.set(FACTORY_INFO, factory_info)
         return tempFolder
 
-
-    def __bobo_traverse__(self, REQUEST, name):
-        """ """
-        # The portal factory intercepts URLs of the form
-        #   .../portal_factory/TYPE_NAME/ID/...
-        # where TYPE_NAME is a type from portal_types.listContentTypes() and
-        # ID is the desired ID for the object.  For intercepted URLs,
-        # portal_factory creates a temporary object of type TYPE_NAME with
-        # id ID and puts it on the traversal stack.  The context for the
-        # temporary object is set to portal_factory's context.
-        #
-        # If the object with id ID already exists in portal_factory's context,
-        # portal_factory returns the existing object.
-        #
-        # All other requests are passed through unchanged.
-        #
-
-        # try to extract a type string from next piece of the URL
-
-        # unmangle type name
-        type_name = urllib.unquote(name)
-        types_tool = getToolByName(self, 'portal_types')
-        # make sure this is really a type name
-        if not type_name in types_tool.listContentTypes():
-            # nope -- do nothing
-            return getattr(self, name)
-        return self.getTempFolder(name)
-
 Globals.InitializeClass(FactoryTool)
