@@ -4,16 +4,15 @@ from OFS.SimpleItem import SimpleItem
 from ZODB.POSException import ConflictError
 
 from Products.CMFCore.utils import UniqueObject, getToolByName
-from Products.CMFCore.CMFCorePermissions import ManagePortal
+from Products.CMFCore.permissions import ManagePortal, View
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
+from Products.CMFPlone.utils import versionTupleFromString
+from Products.CMFPlone.utils import log
+from Products.CMFPlone import transaction
 
 import zLOG
 import traceback
 import sys
-import types
-
-def log(message,summary='',severity=0):
-    zLOG.LOG('Plone: ', severity, summary, message)
 
 _upgradePaths = {}
 _widgetRegistry = {}
@@ -47,7 +46,7 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
     manage_overview = DTMLFile('www/migrationTool', globals())
     manage_results = DTMLFile('www/migrationResults', globals())
     manage_setup = DTMLFile('www/migrationSetup', globals())
-
+    
     # Add a visual note
     def om_icons(self):
         icons = ({
@@ -100,12 +99,23 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
         """ The version this instance of plone is on """
         return self.Control_Panel.Products.CMFPlone.version.lower()
 
+    security.declareProtected(View, 'getFSVersionTuple')
+    def getFSVersionTuple(self):
+        """ returns tuple representing filesystem version """
+        v_str = self.getFileSystemVersion()
+        return versionTupleFromString(v_str)
+
+    security.declareProtected(View, 'getInstanceVersionTuple')
+    def getInstanceVersionTuple(self):
+        """ returns tuple representing instance version """
+        v_str = self.getInstanceVersion()
+        return versionTupleFromString(v_str)
+
     security.declareProtected(ManagePortal, 'needUpgrading')
     def needUpgrading(self):
         """ Need upgrading? """
         return self.getInstanceVersion() != self.getFileSystemVersion()
 
-
     security.declareProtected(ManagePortal, 'coreVersions')
     def coreVersions(self):
         """ Useful core information """
@@ -154,6 +164,15 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
             products[p.id] = product_info
         return products
 
+    security.declareProtected(ManagePortal,'getPILVersion')
+    def getPILVersion(self):
+        """The version of the installed Python Imaging Library."""
+        try:
+            from PIL.Image import VERSION
+        except ImportError:
+            VERSION = None
+        return VERSION
+
     ##############################################################
     # the setup widget registry
     # this is a whole bunch of wrappers
@@ -215,7 +234,10 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
         out = []
         if toAdd: out += self.installItems(widget, toAdd)
         if toDel: out += self.uninstallItems(widget, toDel)
-        return self.manage_results(self, out=out)
+        try:
+            return self.manage_results(self, out=out)
+        except NameError:
+            pass
 
     security.declareProtected(ManagePortal, 'installItems')
     def installItems(self, widget, items):
@@ -280,7 +302,7 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
                     raise
                 else:
                     # abort transaction to safe the zodb
-                    get_transaction().abort()
+                    transaction.abort()
 
         out.append(("End of upgrade path, migration has finished", zLOG.INFO))
 
@@ -323,20 +345,22 @@ class MigrationTool(PloneBaseTool, UniqueObject, SimpleItem):
 
         if dry_run:
             out.append(("Dry run selected, transaction aborted", zLOG.INFO))
-            get_transaction().abort()
+            transaction.abort()
 
         # log all this to the ZLOG
         for msg, sev in out: log(msg, severity=sev)
-
-        return self.manage_results(self, out=out)
+        try:
+            return self.manage_results(self, out=out)
+        except NameError:
+            pass
 
     ##############################################################
     # Private methods
 
     def _check(self):
         """ Are we inside a Plone site?  Are we allowed? """
-        if not hasattr(self,'portal_url'):
-            raise 'You must be in a Plone site to migrate.'
+        if getattr(self,'portal_url', []) == []:
+            raise AttributeError, 'You must be in a Plone site to migrate.'
 
     def _upgrade(self, version):
         version = version.lower()
