@@ -1,5 +1,6 @@
 from __future__ import nested_scopes
-from ComputedAttribute import ComputedAttribute
+import os
+
 from Products.CMFPlone import cmfplone_globals
 from Products.CMFPlone import custom_policies
 from Products.CMFPlone import ToolNames
@@ -20,45 +21,33 @@ def listPolicies(creation=1):
 def addPolicy(label, klass):
     custom_policies[label]=klass
 
-from Products.CMFCore import CMFCorePermissions
+from Products.CMFCore.permissions import AccessContentsInformation, ListFolderContents, ManageProperties, View
 from Products.CMFCore.TypesTool import FactoryTypeInformation
 from Products.CMFCore.DirectoryView import addDirectoryViews
 from Products.CMFCore.utils import getToolByName
 from Products.CMFDefault import Portal, DublinCore
+from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
 from Products.CMFPlone.PloneFolder import OrderedContainer
 import Globals
-import os, sys, re
 
 from AccessControl import ClassSecurityInfo
+from Acquisition import aq_base
+from ComputedAttribute import ComputedAttribute
+from webdav.NullResource import NullResource
+from Products.CMFPlone.PloneFolder import ReplaceableWrapper
+from Products.CMFPlone.utils import log_exc
+from Products.CMFPlone.utils import PACKAGE_HOME
 
-__version__='1.1'
-
-default_frontpage=r"""
-You can customize this frontpage by clicking the edit tab on this document if you
-have the correct permissions. Create folders and put content in those folders.
-Folders will show up in the navigation box if they are published. It's a very
-simple and powerful system.
-
-For more information:
-
-"Plone website":http://www.plone.org
-
-"Zope community":http://www.zope.org
-
-"CMF website":http://cmf.zope.org
-
-There are "mailing lists":http://plone.org/development/lists and
-"recipe websites":http://www.zopelabs.com
-available to provide assistance to you and your new-found Content Management System.
-"Online chat":http://plone.org/development/chat is also a nice way
-of getting advice and help.
-
-Please contribute your experiences at the "Plone website":http://www.plone.org
-
-Thanks for using our product.
-
-"The Plone Team":http://plone.org/about/team
-"""
+default_frontpage="Unable to load front-page skeleton file"
+WWW_DIR = os.path.join(PACKAGE_HOME, 'www')
+try:
+    f = open(os.path.join(WWW_DIR, 'default_frontpage.html'), 'r')
+except IOError:
+    log_exc('Unable to open frontpage skeleton')
+else:
+    default_frontpage = f.read()
+    f.close()
+    del f
 
 member_indexhtml="""\
 member_search=context.restrictedTraverse('member_search_form')
@@ -68,7 +57,7 @@ return member_search()
 factory_type_information = { 'id'             : 'Plone Root'
   , 'meta_type'      : 'Plone Site'
   , 'description'    : """ The portal_type for the root object in a Plone system."""
-  , 'icon'           : 'folder_icon.gif'
+  , 'icon'           : 'site_icon.gif'
   , 'product'        : 'CMFPlone'
   , 'factory'        : 'manage_addSite'
   , 'filter_content_types' : 0
@@ -77,19 +66,35 @@ factory_type_information = { 'id'             : 'Plone Root'
   , 'actions'        : ( { 'id'            : 'view'
                          , 'name'          : 'View'
                          , 'action': 'string:${object_url}'
-                         , 'permissions'   : (CMFCorePermissions.View,)
-                         , 'category'      : 'folder'
+                         , 'permissions'   : (View,)
+                         , 'category'      : 'object'
                          }
                        , { 'id'            : 'edit'
                          , 'name'          : 'Edit'
-                         , 'action': 'string:${object_url}/folder_edit_form'
-                         , 'permissions'   : (CMFCorePermissions.ManageProperties,)
-                         , 'category'      : 'folder'
+                         , 'action': 'string:${object_url}/edit'
+                         , 'permissions'   : (ManageProperties,)
+                         , 'category'      : 'object'
+                         }
+                       , { 'id'            : 'local_roles'
+                         , 'name'          : 'Sharing'
+                         , 'action': 'string:${object_url}/sharing'
+                         , 'permissions'   : (ManageProperties,)
+                         , 'category'      : 'object'
                          }
                        )
+  , 'aliases'        : {
+                        '(Default)'  : '(dynamic view)',
+                        'view'       : '(selected layout)',
+                        'index.html' : '(dynamic view)',
+                        'edit'       : 'folder_edit_form',
+                        'properties' : '',
+                        'sharing'    : 'folder_localrole_form',
+                        'gethtml'    : '',
+                        'mkdir'      : '',
+                       }
   }
 
-class PloneSite(CMFSite, OrderedContainer):
+class PloneSite(CMFSite, OrderedContainer, BrowserDefaultMixin):
     """
     Make PloneSite subclass CMFSite and add some methods.
     This will be useful for adding more things later on.
@@ -97,13 +102,44 @@ class PloneSite(CMFSite, OrderedContainer):
     security=ClassSecurityInfo()
     meta_type = portal_type = 'Plone Site'
     __implements__ = DublinCore.DefaultDublinCoreImpl.__implements__ + \
-                     OrderedContainer.__implements__
+                     OrderedContainer.__implements__ + \
+                     BrowserDefaultMixin.__implements__
+
+    manage_renameObject = OrderedContainer.manage_renameObject
+
+    moveObject = OrderedContainer.moveObject
+    moveObjectsByDelta = OrderedContainer.moveObjectsByDelta
+
+    # Switch off ZMI ordering interface as it assumes a slightly
+    # different functionality
+    has_order_support = 0
+    manage_main = Globals.DTMLFile('www/main', globals())
 
     def __browser_default__(self, request):
         """ Set default so we can return whatever we want instead
         of index_html """
         return getToolByName(self, 'plone_utils').browserDefault(self)
 
+    def index_html(self):
+        """ Acquire if not present. """
+        request = getattr(self, 'REQUEST', None)
+        if request and request.has_key('REQUEST_METHOD'):
+            if request.maybe_webdav_client:
+                method = request['REQUEST_METHOD']
+                if method in ('PUT',):
+                    # Very likely a WebDAV client trying to create something
+                    return ReplaceableWrapper(NullResource(self, 'index_html'))
+                elif method in ('GET', 'HEAD', 'POST'):
+                    # Do nothing, let it go and acquire.
+                    pass
+                else:
+                    raise AttributeError, 'index_html'
+        # Acquire from skin.
+        _target = self.__getattr__('index_html')
+        return ReplaceableWrapper(aq_base(_target).__of__(self))
+
+    index_html = ComputedAttribute(index_html, 1)
+
     def manage_beforeDelete(self, container, item):
         """ Should send out an Event before Site is being deleted """
         self.removal_inprogress=1
@@ -124,6 +160,25 @@ class PloneSite(CMFSite, OrderedContainer):
 
     management_page_charset = ComputedAttribute(_management_page_charset, 1)
 
+    def view(self):
+        """ Ensure that we get a plain view of the object, via a delegation to
+        __call__(), which is defined in BrowserDefaultMixin
+        """
+        return self()
+
+    security.declareProtected(AccessContentsInformation,
+			     'folderlistingFolderContents')
+    def folderlistingFolderContents(self, spec=None, contentFilter=None):
+        """Calls listFolderContents in protected only by ACI so that
+        folder_listing can work without the List folder contents permission,
+        as in CMFDefault.
+
+        This is copied from Archetypes Basefolder and is needed by the
+        reference browser.
+        """
+        return self.listFolderContents(spec, contentFilter)
+
+
 Globals.InitializeClass(PloneSite)
 
 class PloneGenerator(Portal.PortalGenerator):
@@ -150,14 +205,9 @@ class PloneGenerator(Portal.PortalGenerator):
                 typeObj=getattr(typesTool, typeInfo.getId())
                 view='folder_contents'
                 typeObj._setPropValue('immediate_view', view)
-                _actions=typeInfo._cloneActions()
-                for action in _actions:
-                    if action.id=='edit':
-                        action.title='Properties'
-                typeObj._actions=_actions
 
     def customizePortalOptions(self, p):
-        p.manage_permission( CMFCorePermissions.ListFolderContents, \
+        p.manage_permission( ListFolderContents, \
                              ('Manager', 'Member', 'Owner',), acquire=1 )
         p.portal_skins.default_skin='Plone Default'
         p.portal_skins.allow_any=0 # Skin changing for users is turned off by default
@@ -174,22 +224,24 @@ class PloneGenerator(Portal.PortalGenerator):
         members = getattr(p , 'Members')
         members.setTitle('Members')
         members.setDescription("Container for portal members' home directories")
-        catalog.unindexObject(members) #unindex Members folder
+        ##catalog.unindexObject(members) #unindex Members folder
 
-        # add index_html to portal root
-        p.invokeFactory('Document', 'index_html')
-        idx = getattr(p, 'index_html')
+        # add front page to portal root
+        p.invokeFactory('Document', 'front-page')
+        idx = getattr(p, 'front-page')
         idx.setTitle('Welcome to Plone')
-        idx.setDescription('This welcome page is used to introduce you'+\
-                         ' to the Plone Content Management System.')
-        idx.setFormat('structured-text')
+        idx.setDescription('Congratulations! You have successfully'+\
+                         ' installed Plone.')
+        idx.setFormat('html')
         if idx.meta_type == 'Document':
             # CMFDefault document
-            idx.edit('structured-text', default_frontpage)
+            idx.edit('html', default_frontpage)
         else:
             idx.edit(text=default_frontpage)
         idx.reindexObject()
 
+        p.setDefaultPage('front-page')
+
         # add index_html to Members area
         addPy = members.manage_addProduct['PythonScripts'].manage_addPythonScript
         addPy('index_html')
@@ -271,109 +323,66 @@ class PloneGenerator(Portal.PortalGenerator):
         path=','.join(path)
         sk_tool.addSkinSelection('Plone Default', path)
 
-#  XXX removed these, since they are no longer shipping with Plone
-#
-#        self.setupSecondarySkin(sk_tool, 'Plone Autumn',
-#                                'plone_styles/autumn')
-#        self.setupSecondarySkin(sk_tool, 'Plone Core',
-#
-#        self.setupSecondarySkin(sk_tool, 'Plone Core Inverted',
-#                                'plone_styles/core_inverted')
-#        self.setupSecondarySkin(sk_tool, 'Plone Corporate',
-#                                'plone_styles/corporate')
-#        self.setupSecondarySkin(sk_tool, 'Plone Greensleeves',
-#                                'plone_styles/greensleeves')
-#        self.setupSecondarySkin(sk_tool, 'Plone Kitty',
-#                                'plone_styles/kitty')
-#        self.setupSecondarySkin(sk_tool, 'Plone Mozilla',
-#                                'plone_styles/mozilla')
-#        self.setupSecondarySkin(sk_tool, 'Plone Mozilla New',
-#                                'plone_styles/mozilla_new')
-#        self.setupSecondarySkin(sk_tool, 'Plone Prime',
-#                                'plone_styles/prime')
-#        self.setupSecondarySkin(sk_tool, 'Plone Zed',
-#                                'plone_styles/zed')
-
         addDirectoryViews( sk_tool, 'skins', cmfplone_globals )
 
-    def setupForms(self, p):
-        """ This is being deprecated.  Please see CMFFormController """
-
+    def setupNavTree(self,p):
+        ''' sets up the default propertysheet for the navtree '''
         prop_tool = p.portal_properties
-        prop_tool.manage_addPropertySheet('navigation_properties', \
-                                          'Navigation Properties')
-        prop_tool.manage_addPropertySheet('form_properties', 'Form Properties')
+        prop_tool.manage_addPropertySheet('navtree_properties', 'NavigationTree properties')
 
-        form_tool = p.portal_form
-        form_tool.setValidators('link_edit_form', \
-                                ['validate_id', 'validate_link_edit'])
-        form_tool.setValidators('newsitem_edit_form', \
-                                ['validate_id', 'validate_newsitem_edit'])
-        form_tool.setValidators('document_edit_form', \
-                                ['validate_id', 'validate_document_edit'])
-        form_tool.setValidators('image_edit_form', \
-                                ['validate_id', 'validate_image_edit'])
-        form_tool.setValidators('file_edit_form', \
-                                ['validate_id', 'validate_file_edit'])
-        form_tool.setValidators('folder_edit_form', \
-                                ['validate_id', 'validate_folder_edit'])
-        form_tool.setValidators('event_edit_form', \
-                                ['validate_id', 'validate_event_edit'])
-        form_tool.setValidators('topic_edit_form', \
-                                ['validate_id', 'validate_topic_edit'])
-        form_tool.setValidators('content_status_history', \
-                                ['validate_content_status_modify'])
-        form_tool.setValidators('reconfig_form', \
-                                ['validate_reconfig'])
-        form_tool.setValidators('personalize_form', \
-                                ['validate_personalize'])
-        form_tool.setValidators('join_form', \
-                                ['validate_registration'])
-        form_tool.setValidators('metadata_edit_form', \
-                                ['validate_metadata_edit'])
-        form_tool.setValidators('sendto_form', \
-                                ['validate_sendto'])
-        form_tool.setValidators('folder_rename_form', \
-                                ['validate_folder_rename'])
+        ntp=prop_tool.navtree_properties
+        navtree_bl=['ATBooleanCriterion',
+                    'ATCurrentAuthorCriterion',
+                    'ATPathCriterion',
+                    'ATDateCriteria',
+                    'ATDateRangeCriterion',
+                    'ATListCriterion',
+                    'ATPortalTypeCriterion',
+                    'ATReferenceCriterion',
+                    'ATSelectionCriterion',
+                    'ATSimpleIntCriterion',
+                    'ATSimpleStringCriterion',
+                    'ATSortCriterion',
+                    'Discussion Item',
+                    'Plone Site',
+                    'TempFolder']
+        ntp._setProperty('typesNotToList', navtree_bl, 'lines')
+        ntp._setProperty('sortAttribute', 'getObjPositionInParent', 'string')
+        ntp._setProperty('sortOrder', 'asc', 'string')
+        ntp._setProperty('sitemapDepth', 3, 'int')
+        ntp._setProperty('includeTop', 1, 'boolean')
 
-        #set up properties for StatelessTreeNav
-        from Products.CMFPlone.StatelessTreeNav \
-             import setupNavTreePropertySheet
-        setupNavTreePropertySheet(prop_tool)
+        # TODO: needs to be supported
+        ntp._setProperty('topLevel', 0, 'int')
+        ntp._setProperty('idsNotToList', [] , 'lines')
+        ntp._setProperty('skipIndex_html',1,'boolean')
+        ntp._setProperty('showAllParents',1,'boolean')
 
-        # grab the initial portal navigation properties
-        # from data/navigation_properties
-        nav_tool = getToolByName(p, 'portal_navigation')
-
-        # open and parse the file
-        filename='navigation_properties'
-        src_file = open(os.path.join(Globals.package_home(globals()),
-                                     'data', filename), 'r')
-        src_lines = src_file.readlines()
-        src_file.close()
-
-        re_comment = re.compile(r"\s*#")
-        re_blank = re.compile(r"\s*\n")
-        re_transition = re.compile(r"\s*(?P<type>[^\.]*)\.(?P<page>[^\.]*)\.(?P<outcome>[^\s]*)\s*=\s*(?P<action>[^$]*)$")
-        for line in src_lines:
-            line = line.strip()
-            if not re_comment.match(line) and not re_blank.match(line):
-                match = re_transition.match(line)
-                if match:
-                    nav_tool.addTransitionFor(match.group('type'),
-                                              match.group('page'),
-                                              match.group('outcome'),
-                                              match.group('action'))
-                else:
-                    sys.stderr.write("Unable to parse '%s' in navigation properties file" % (line))
+        # Canditates to be implemented
+        ntp._setProperty('showMyUserFolderOnly', 1, 'boolean')
+        ntp._setProperty('showFolderishSiblingsOnly', 1, 'boolean')
+        ntp._setProperty('showFolderishChildrenOnly', 1, 'boolean')
+        ntp._setProperty('showNonFolderishObject', 0, 'boolean')
+        ntp._setProperty('showTopicResults', 1, 'boolean')
+        ntp._setProperty('rolesSeeUnpublishedContent', ['Manager','Reviewer','Owner'] , 'lines')
+        ntp._setProperty('sortCriteria', ['isPrincipiaFolderish,desc']  , 'lines')
+        ntp._setProperty('parentMetaTypesNotToQuery',['TempFolder'],'lines')
 
+        # The following properties will not be supported anymore         
+        ntp._setProperty('batchSize', 30, 'int')
+        ntp._setProperty('croppingLength',256,'int')
+        ntp._setProperty('forceParentsInBatch',0,'boolean')
+        ntp._setProperty('rolesSeeContentsView', ['Manager','Reviewer','Owner'] , 'lines')
+        ntp._setProperty('rolesSeeHiddenContent', ['Manager',] , 'lines')
+        ntp._setProperty('bottomLevel', 65535 , 'int')
+ 
     def setupPlone(self, p):
         self.customizePortalTypes(p)
         self.customizePortalOptions(p)
         self.setupPloneWorkflow(p)
         self.setupPloneSkins(p)
         self.setupPortalContent(p)
-        self.setupForms(p)
+        self.setupNavTree(p)
 
         m = p.portal_migration
 
@@ -409,9 +418,7 @@ class PloneGenerator(Portal.PortalGenerator):
         addCMFPloneTool(ToolNames.SyndicationTool, None)
 
         addCMFPloneTool(ToolNames.UtilsTool, None)
-        addCMFPloneTool(ToolNames.NavigationTool, None)
         addCMFPloneTool(ToolNames.FactoryTool, None)
-        addCMFPloneTool(ToolNames.FormTool, None)
         addCMFPloneTool(ToolNames.MigrationTool, None)
         addCMFPloneTool(ToolNames.ActionIconsTool, None)
         addCMFPloneTool(ToolNames.CalendarTool, None)
@@ -438,7 +445,7 @@ def manage_addSite(self, id, title='Portal', description='',
                    create_userfolder=1,
                    email_from_address='postmaster@localhost',
                    email_from_name='Portal Administrator',
-                   validate_email=0,
+                   validate_email=1,
                    custom_policy='Default Plone',
                    RESPONSE=None):
     """ Plone Site factory """
