@@ -1,37 +1,19 @@
 import sys
+import os
 import Globals
-from os import path
-from Acquisition import aq_base
 
 cmfplone_globals = globals()
 this_module = sys.modules[ __name__ ]
+_marker = []
 
 # Stores the available 'Customization Policies'
 custom_policies = {}
 
 ADD_CONTENT_PERMISSION = 'Add portal content'
 
-misc_ = {'plone_icon': Globals.ImageFile(path.join('skins',
-                                                   'plone_images',
-                                                   'logoIcon.gif'),
-                                         cmfplone_globals)}
-
-# For plone_debug method
-import zLOG
-
-
-def log(message,summary='',severity=0):
-    zLOG.LOG('MyDebugLog',severity,summary,message)
-
-def transaction_note(note):
-    """ Write human legible note """
-    T=get_transaction()
-    T.note(str(note))
-
-def base_hasattr(ob, name):
-    ob = aq_base(ob)
-    return hasattr(ob, name)
-
+misc_ = {'plone_icon': Globals.ImageFile(
+                       os.path.join('skins', 'plone_images', 'logoIcon.gif'),
+                       cmfplone_globals)}
 
 def initialize(context):
 
@@ -41,21 +23,25 @@ def initialize(context):
     # For test authors (and people who use parts of Plone only)
     # it does speed up import *significantly*.
 
-    from AccessControl import ModuleSecurityInfo, ClassSecurityInfo
-    from AccessControl import allow_module, allow_class, allow_type
+    from AccessControl import ModuleSecurityInfo
+    from AccessControl import allow_module, allow_class
+
+    # zLOG is deprecated in Zope > 2.7.0, use logger instead
+    ModuleSecurityInfo('logging').declarePublic('getLogger')
+    from logging import Logger
+    allow_class(Logger)
 
     ModuleSecurityInfo('zLOG').declarePublic('LOG')
     ModuleSecurityInfo('zLOG').declarePublic('INFO')
-    ModuleSecurityInfo('Products.CMFPlone.PloneUtilities').declarePublic('translate_wrapper')
-    ModuleSecurityInfo('Products.CMFPlone.PloneUtilities').declarePublic('localized_time')
+    ModuleSecurityInfo('zLOG').declarePublic('WARNING')
+
+    allow_module('Products.CMFPlone.utils')
+
+    # This is now deprecated, use utils instead.
     allow_module('Products.CMFPlone.PloneUtilities')
 
-    import StatelessTreeNav
-    from StatelessTree import NavigationTreeViewBuilder
-    allow_class(NavigationTreeViewBuilder)
-
     # For form validation bits
-    from PloneUtilities import IndexIterator
+    from Products.CMFPlone.utils import IndexIterator
     allow_class(IndexIterator)
 
     # Make IndexIterator available at module level
@@ -63,7 +49,7 @@ def initialize(context):
 
     # For content_status_modify
     from Products.CMFCore.WorkflowCore import ObjectMoved, ObjectDeleted, \
-         WorkflowException
+                                              WorkflowException
     ModuleSecurityInfo('Products.CMFCore.WorkflowCore').declarePublic('ObjectMoved')
     ModuleSecurityInfo('Products.CMFCore.WorkflowCore').declarePublic('ObjectDeleted')
     ModuleSecurityInfo('Products.CMFCore.WorkflowCore').declarePublic('WorkflowException')
@@ -80,53 +66,64 @@ def initialize(context):
     from StringIO import StringIO
     allow_class(StringIO)
 
-    ModuleSecurityInfo('Products.CMFPlone').declarePublic('transaction_note')
+    # Make listPolicies importable TTW
     ModuleSecurityInfo('Products.CMFPlone.Portal').declarePublic('listPolicies')
 
-    # make Unauthorized importable ttw
+    # Make Unauthorized importable TTW
     ModuleSecurityInfo('AccessControl').declarePublic('Unauthorized')
 
-    # make ConflictError importable ttw
+    # Make ConflictError importable TTW
     ModuleSecurityInfo('ZODB.POSException').declarePublic('ConflictError')
 
-    # make ZCTextIndex ParseError importable ttw
+    # Make ZCTextIndex ParseError importable TTW
     ModuleSecurityInfo('Products.ZCTextIndex.ParseTree').declarePublic('ParseError')
 
-    # make base_hasattr importable ttw
-    ModuleSecurityInfo('Products.CMFPlone').declarePublic('base_hasattr')
+    # Make DateTimeError importable TTW
+    ModuleSecurityInfo('DateTime.DateTime').declarePublic('DateTimeError')
+    ModuleSecurityInfo('DateTime.DateTime').declarePublic('SyntaxError')
 
-    # Formulator
-    ModuleSecurityInfo('Products.Formulator').declarePublic('StringField', 'EmailField')
-    ModuleSecurityInfo('Products.Formulator.Form').declarePublic('FormValidationError', 'BasicForm')
+    # Make CopyError importable TTW
+    ModuleSecurityInfo('OFS.CopySupport').declarePublic('CopyError')
 
-    from Products.Formulator.StandardFields import StringField, EmailField
-    from Products.Formulator.Form import FormValidationError, BasicForm
-    allow_class(StringField)
-    allow_class(EmailField)
-    allow_class(FormValidationError)
-    allow_class(BasicForm)
+    # Make DiscussionNotAllowed importable TTW
+    ModuleSecurityInfo('Products.CMFDefault.DiscussionTool').declarePublic('DiscussionNotAllowed')
+    
+    # Make AllowSendto importable TTW
+    ModuleSecurityInfo('Products.CMFPlone.PloneTool').declarePublic('AllowSendto')
+
+    # Make ZCatalog's mergeResults importable TTW
+    ModuleSecurityInfo('Products.ZCatalog.Catalog').declarePublic('mergeResults')
+
+    # Backward compatibility only, please import from utils directly
+    from Products.CMFPlone.utils import transaction_note, base_hasattr
+    this_module.transaction_note = transaction_note
+    this_module.base_hasattr = base_hasattr
+
+    # Make the navtree constructs available TTW
+    allow_module('Products.CMFPlone.browser.navtree')
 
     # Setup migrations
     import migrations
+    migrations.executeMigrations()
     migrations.registerMigrations()
 
-    import setup           # Configuration Machinery - Andy we need to fix
-                           # this at some point
-    import imagePatch      # WAII and 508 we need more properties on image
-                           # objects
-    import zserverPatch    # identify Plone in HTTP Headers - netcraft
-                           # here we come!
-    import UnicodeSplitter # registers unicode splitter w/ zctextindex
-                           # pipeline registry
-    import setFormatPatch  # patch DefaultDublinCoreImpl.setFormat to work
-                           # around http://plone.org/collector/1323
-    import verifyObjectPastePatch   # patch PortalFolder to work around
-                                    # http://plone.org/collector/2183
+    # Inititalize configuration machinery 
+    import setup
 
+    # Apply monkey patches
+    import patches
+
+    # Register unicode splitter w/ ZCTextIndex
+    # pipeline registry
+    import UnicodeSplitter
+
+    # Register Plone skins directory
     from Products.CMFCore import DirectoryView
     DirectoryView.registerDirectory('skins', cmfplone_globals)
 
-    import PloneContent, PloneFolder, PloneWorkflow, FolderWorkflow, Portal
+    # Plone content
+    import PloneContent, PloneFolder, PloneWorkflow, FolderWorkflow
+    import Portal
 
     contentClasses = ( PloneFolder.PloneFolder , )
     contentConstructors = ( PloneFolder.addPloneFolder, )
@@ -142,27 +139,26 @@ def initialize(context):
         contentConstructors += ( LargePloneFolder.addLargePloneFolder,)
         ftis += (LargePloneFolder.factory_type_information, )
 
-    # CMFCore and CMFDefault Tools
+    # CMFCore and CMFDefault tools
     from Products.CMFCore import CachingPolicyManager
 
-    # Plone Tools
-    import PloneTool, NavigationTool, FactoryTool
-    import FormTool, InterfaceTool, MigrationTool, PloneControlPanel
+    # Plone tools
+    import PloneTool, FactoryTool
+    import InterfaceTool, MigrationTool, PloneControlPanel
     import MembershipTool, WorkflowTool, URLTool, MetadataTool
     import RegistrationTool, MemberDataTool, SyndicationTool
     import PropertiesTool, ActionsTool, TypesTool, UndoTool
     import CatalogTool, SkinsTool, DiscussionTool
     import CalendarTool, ActionIconsTool, QuickInstallerTool
     import GroupDataTool, GroupsTool
+    import TranslationServiceTool
 
     tools = ( MembershipTool.MembershipTool,
               MemberDataTool.MemberDataTool,
               PloneTool.PloneTool,
               WorkflowTool.WorkflowTool,
               CachingPolicyManager.CachingPolicyManager,
-              NavigationTool.NavigationTool,
               FactoryTool.FactoryTool,
-              FormTool.FormTool,
               PropertiesTool.PropertiesTool,
               MigrationTool.MigrationTool,
               InterfaceTool.InterfaceTool,
@@ -182,30 +178,45 @@ def initialize(context):
               QuickInstallerTool.QuickInstallerTool,
               GroupsTool.GroupsTool,
               GroupDataTool.GroupDataTool,
+              TranslationServiceTool.TranslationServiceTool,
             )
 
-    from Products.CMFCore import utils
-    from PloneUtilities import ToolInit
-    import Portal
+    from Products.CMFCore.utils import initializeBasesPhase1
+    from Products.CMFCore.utils import initializeBasesPhase2
+    from Products.CMFCore.utils import ContentInit
+    from Products.CMFPlone.utils import ToolInit
 
-    z_bases = utils.initializeBasesPhase1(contentClasses, this_module)
-    utils.initializeBasesPhase2( z_bases, context )
+    # Register tools, content, and customization policies
+    z_bases = initializeBasesPhase1(contentClasses, this_module)
+    initializeBasesPhase2(z_bases, context)
 
-    ToolInit('Plone Tool', tools=tools,
-             product_name='CMFPlone', icon='tool.gif',
+    ToolInit('Plone Tool'
+             , tools=tools
+             , product_name='CMFPlone'
+             , icon='tool.gif'
              ).initialize( context )
 
-    utils.ContentInit( 'Plone Content'
-                     , content_types=contentClasses
-                     , permission=ADD_CONTENT_PERMISSION
-                     , extra_constructors=contentConstructors
-                     , fti=ftis
-                     ).initialize( context )
+    ContentInit('Plone Content'
+                , content_types=contentClasses
+                , permission=ADD_CONTENT_PERMISSION
+                , extra_constructors=contentConstructors
+                , fti=ftis
+                ).initialize( context )
 
     Portal.register(context, cmfplone_globals)
 
     import CustomizationPolicy
-    import PrivateSitePolicy
-
     CustomizationPolicy.register(context, cmfplone_globals)
-    PrivateSitePolicy.register(context, cmfplone_globals)
+
+    ModuleSecurityInfo('Products.CMFPlone').declarePrivate('transaction')
+
+
+# Zope 2.8-style transaction module
+# BBB: Zope 2.7
+try:
+    import Zope2
+except ImportError:
+    import transaction_ as transaction
+else:
+    import transaction
+
