@@ -15,6 +15,5 @@ import release_candidates
 # Plone2 Final to 2.0.1
 import final_two01
 
-import two03_two04
-
+# Plone 2.0.4 to 2.0.5
 import two04_two05
