@@ -17,8 +17,7 @@ from Products.CMFCore.Expression import Expression
 from Products.CMFPlone.migrations.v2.plone2_base import setupExtEditor, addDocumentActions, addActionIcons
 from Products.CMFPlone.migrations.migration_util import safeEditProperty, addLinesToProperty
 from Products.CMFPlone.setup import ConfigurationMethods
-from Products.CMFPlone import ToolNames
-from Products.CMFPlone.StatelessTreeNav import setupNavTreePropertySheet
+from Products.CMFPlone import ToolNames, transaction
 from portlet_migration import upgradeSlots2Portlets
 import plone2_base
 
@@ -87,9 +86,6 @@ def oneX_twoBeta2(portal):
     out.append("Adding in new form controller")
     addFormController(portal)
 
-    out.append("Adding in site actions")
-    ConfigurationMethods.addSiteActions(portal, portal)
-
     out.append("Moving portraits into the memberdata tool")
     #migrate Memberdata, Membership and Portraits
     migrateMemberdataTool(portal)
@@ -186,18 +182,18 @@ def migrateTools(portal):
     _migrate(portal, 'portal_discussion', ToolNames.DiscussionTool, ['_actions'])
     _migrate(portal, 'portal_groups', ToolNames.GroupsTool, ['_actions',
         'groupworkspaces_id', 'groupWorkspacesCreationFlag',
-        'groupWorkspaceType', ]) # XXX 'groupworkspaces_title', 'groupWorkspaceContainerType'
+        'groupWorkspaceType', ])
     _migrate(portal, 'portal_groupdata', ToolNames.GroupDataTool, ['_actions',
-        '_members']) # XXX properties
+        '_members'])
 
-    orig=_migrate(portal, 'portal_types', ToolNames.TypesTool, ['_actions', 'meta_types']) #XXX
+    orig=_migrate(portal, 'portal_types', ToolNames.TypesTool, ['_actions', 'meta_types'])
     tt = getToolByName(portal, 'portal_types')
     for info in orig._objects:
         obj = aq_base(getattr(orig, info['id']))
         tt._setObject(info['id'], obj)
 
     avail_paths=manage_listAvailableDirectories()
-    orig = _migrate(portal, 'portal_skins', ToolNames.SkinsTool, ['selections', '_actions']) #XXX
+    orig = _migrate(portal, 'portal_skins', ToolNames.SkinsTool, ['selections', '_actions'])
     st = getToolByName(portal, 'portal_skins')
     for info in orig._objects:
         obj = aq_base(getattr(orig, info['id']))
@@ -356,7 +352,7 @@ def addGroupUserFolder(portal):
         NOTE: In the Install routine for GRUF it does a subtransaction commit()
         so that you can manipulate the acl_users folders.
     """
-    get_transaction().commit(1)
+    transaction.savepoint(optimistic=True)
 
     qi=getToolByName(portal, 'portal_quickinstaller')
     addGRUFTool=portal.manage_addProduct['GroupUserFolder'].manage_addTool
@@ -389,8 +385,8 @@ def addFormController(portal):
         qi.notifyInstalled('CMFFormController') #Portal.py got to it first
 
 def addActionIcons(portal):
-    """ After installing QuickInstaller.  We must commit(1) a subtrnx
-        so that we will be able to addActionIcons() to the tool
+    """ After installing QuickInstaller.  We must savepoint(optimistic=True) a
+        subtrnx so that we will be able to addActionIcons() to the tool
     """
     try:
         qi=getToolByName(portal, 'portal_quickinstaller')
@@ -496,7 +492,7 @@ def fixUndoActions(portal):
             del action
     st.addAction('undo',
                  'Undo',
-                 'string: ${portal_url}/undo_form',
+                 'string:${portal_url}/undo_form',
                  'member',
                  'List undoable changes',
                  'user')
