@@ -1,6 +1,9 @@
-import dependencies
+import os
 
-from Products.CMFPlone import MigrationTool #:wort registerSetupWidget
+if not os.environ.get('ZOPETESTCASE'):
+    import dependencies
+
+from Products.CMFPlone import MigrationTool
 from customization_policy import CustomizationPolicySetup
 from ConfigurationMethods import GeneralSetup
 
