@@ -9,6 +9,6 @@
 ##
 request=context.REQUEST
 
-context.plone_log('copy_to_zlog='+str(copy_to_zlog))
-results = context.error_log.setProperties(keep_entries,copy_to_zlog,ignored_exceptions)
-return request.RESPONSE.redirect(context.absolute_url() +'/prefs_error_log_form?portal_status_message=%s' % ('Data+Updated'))
+context.error_log.setProperties(keep_entries,copy_to_zlog,ignored_exceptions)
+return request.RESPONSE.redirect(context.absolute_url() +
+            '/prefs_error_log_form?portal_status_message=%s' % ('Changes+made.'))
