@@ -9,14 +9,19 @@
 ##
 if obj is None:
     obj=context
-		
+
 parent = obj.inReplyTo()
 if parent is not None:
     talkback = context.portal_discussion.getDiscussionFor(parent)
 else:
     talkback = parent = obj.aq_parent
 
+# remove the discussion item
 talkback.deleteReply( obj.getId() )
 
-context.REQUEST['RESPONSE'].redirect( parent.absolute_url()
-         + '?portal_status_message=Reply+deleted' )
+# redirect to the object that is being discussed
+redirect_target = context.plone_utils.getDiscussionThread(talkback)[0]
+view = redirect_target.getTypeInfo().getActionById('view')
+
+context.REQUEST['RESPONSE'].redirect( redirect_target.absolute_url()
+         + '/%s?portal_status_message=Reply+deleted' % view )
