@@ -7,5 +7,7 @@
 ##parameters=
 ##title=http://cmf.zope.org/Members/tseaver/20020723_external_editor_available
 
-return context.REQUEST['RESPONSE'].redirect('%s/externalEdit_/%s' % \
-    (context.aq_parent.absolute_url(), context.getId()) )
+from Products.PythonScripts.standard import url_quote
+return context.REQUEST['RESPONSE'].redirect(
+    '%s/externalEdit_/%s' % (context.aq_parent.absolute_url(),
+                             url_quote(context.getId())))
