@@ -16,4 +16,4 @@ if month==0 or month==1:
 else:
     month-=1
 
-return DateTime('%s/1/%s' % (month, year))
+return DateTime(year, month, 1)
