@@ -11,11 +11,11 @@ from Products.CMFPlone.tests import PloneTestCase
 from Products.CMFPlone.tests import dummy
 
 AddPortalTopics = 'Add portal topics'
-from DateTime import DateTime
 from Products.CMFPlone import LargePloneFolder
+from Products.CMFPlone import transaction
+from OFS.CopySupport import CopyError
 
-
-#XXX NOTE
+#    NOTE
 #    document, link, and newsitem edit's are now validated
 #    so we must pass in fields that the validators need
 #    such as title on a favorite's link_edit
@@ -27,8 +27,6 @@ class TestContentTypeScripts(PloneTestCase.PloneTestCase):
         self.setPermissions(perms + [AddPortalTopics], 'Member')
         self.discussion = self.portal.portal_discussion
         self.request = self.app.REQUEST
-        # Don't pay for catalog maintenance
-        self.portal.manage_delObjects('portal_catalog')
 
     def getPermissionsOfRole(self, role):
         perms = self.portal.permissionsOfRole(role)
@@ -100,21 +98,21 @@ class TestContentTypeScripts(PloneTestCase.PloneTestCase):
 
     def testFileCreate(self):
         self.folder.invokeFactory('File', id='file', file=dummy.File())
-        self.assertEqual(str(self.folder.file), 'file data')
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
 
     def testFileEdit(self):
         self.folder.invokeFactory('File', id='file')
         self.folder.file.file_edit(file=dummy.File())
-        self.assertEqual(str(self.folder.file), 'file data')
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
 
     def testImageCreate(self):
-        self.folder.invokeFactory('Image', id='image', file=dummy.File())
-        self.assertEqual(str(self.folder.image.data), 'file data')
+        self.folder.invokeFactory('Image', id='image', file=dummy.Image())
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
 
     def testImageEdit(self):
         self.folder.invokeFactory('Image', id='image')
-        self.folder.image.image_edit(file=dummy.File())
-        self.assertEqual(str(self.folder.image.data), 'file data')
+        self.folder.image.image_edit(file=dummy.Image())
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
 
     def testFolderCreate(self):
         self.folder.invokeFactory('Folder', id='folder', title='Foo', description='Bar')
@@ -160,18 +158,42 @@ class TestContentTypeScripts(PloneTestCase.PloneTestCase):
         self.assertEqual(self.folder.newsitem.EditableBody(), 'data')
         self.assertEqual(self.folder.newsitem.Title(), 'Foo')
 
-    def testTopicCreate(self):
-        self.folder.invokeFactory('Topic', id='topic', title='Foo')
-        self.assertEqual(self.folder.topic.Title(), 'Foo')
+    # Bug tests
 
-    def testTopicEditTopic(self):
-        self.folder.invokeFactory('Topic', id='topic')
-        self.folder.topic.topic_editTopic(1, 'topic', title='Foo')
-        self.assertEqual(self.folder.topic.Title(), 'Foo')
+    def testClearImageTitle(self):
+        # Test for http://plone.org/collector/3303
+        # Should be able to clear Image title
+        self.folder.invokeFactory('Image', id='image', title='Foo', file=dummy.Image())
+        self.assertEqual(self.folder.image.Title(), 'Foo')
+        self.folder.image.image_edit(title='')
+        self.assertEqual(self.folder.image.Title(), '')
+ 
+    def testAddToFavorites(self):
+        # Ugh, addFavorite traverses to remote_url, so make sure it can.
+        self.setRoles(['Manager'])
+        self.portal.invokeFactory('Folder', id='bar')
+        self.portal.bar.invokeFactory('Document', id='baz.html')
+        self.setRoles(['Member'])
+        # back to normal
+        self.failIf('Favorites' in self.folder.objectIds())
+        self.portal.bar['baz.html'].addtoFavorites()
+        self.failUnless('Favorites' in self.folder.objectIds())
+        favorite = self.folder.Favorites.objectValues()[0]
+        self.assertEqual(favorite.getRemoteUrl(),
+                         '%s/bar/baz.html' % self.portal.portal_url())
+        # Make sure the script created AT types
+        self.assertEqual(self.folder.Favorites.meta_type, 'ATFolder')
+        self.assertEqual(favorite.meta_type, 'ATFavorite')
+        # Make sure that the script gave a Title to the folder
+        self.assertEqual(self.folder.Favorites.Title(), 'Favorites')
 
-    #def testTopicEditCriteria(self):
-    #    self.folder.invokeFactory('Topic', id='topic')
-    #    # TODO: Analyze that funky data structure
+    def test_listMetaTypes(self):
+        self.folder.invokeFactory('Document', id='doc')
+        tool = self.portal.plone_utils
+        doc = self.folder.doc
+        doc.setTitle('title')
+        metatypes = tool.listMetaTags(doc)
+        # TODO: atm it checks only of the script can be called w/o an error
 
 
 class TestEditShortName(PloneTestCase.PloneTestCase):
@@ -180,47 +202,51 @@ class TestEditShortName(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.folder.invokeFactory('File', id='file', file=dummy.File())
-        self.folder.invokeFactory('Image', id='image', file=dummy.File())
+        self.folder.invokeFactory('Image', id='image', file=dummy.Image())
 
     def testFileEditNone(self):
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
         self.folder.file.file_edit(file=None, title='Foo')
         self.assertEqual(self.folder.file.Title(), 'Foo')
         # Data is not changed
-        self.assertEqual(str(self.folder.file), 'file data')
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
 
     def testImageEditNone(self):
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
         self.folder.image.image_edit(file=None, title='Foo')
         self.assertEqual(self.folder.image.Title(), 'Foo')
         # Data is not changed
-        self.assertEqual(str(self.folder.image.data), 'file data')
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
 
     def testFileEditEmptyString(self):
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
         self.folder.file.file_edit(file='', title='Foo')
         self.assertEqual(self.folder.file.Title(), 'Foo')
         # Data is not changed
-        self.assertEqual(str(self.folder.file), 'file data')
+        self.assertEqual(str(self.folder.file), dummy.TEXT)
 
     def testImageEditEmptyString(self):
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
         self.folder.image.image_edit(file='', title='Foo')
         self.assertEqual(self.folder.image.Title(), 'Foo')
         # Data is not changed
-        self.assertEqual(str(self.folder.image.data), 'file data')
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
 
     def testFileEditString(self):
         self.folder.file.file_edit(file='foo')
-        self.assertEqual(str(self.folder.file), 'foo')
+        self.assertEqual(str(self.folder.file.getFile()), 'foo')
 
     def testImageEditString(self):
-        self.folder.image.image_edit(file='foo')
-        self.assertEqual(str(self.folder.image.data), 'foo')
+        self.folder.image.image_edit(file=dummy.GIF)
+        self.assertEqual(str(self.folder.image.data), dummy.GIF)
 
     def testFileEditShortName(self):
-        get_transaction().commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.file.file_edit(id='fred')
         self.failUnless('fred' in self.folder.objectIds())
 
     def testImageEditShortName(self):
-        get_transaction().commit(1) # make rename work
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.image.image_edit(id='fred')
         self.failUnless('fred' in self.folder.objectIds())
 
@@ -233,39 +259,76 @@ class TestEditFileKeepsMimeType(PloneTestCase.PloneTestCase):
         self.folder.invokeFactory('File', id='file')
         self.folder.file.file_edit(file=dummy.File('foo.pdf'))
         self.folder.invokeFactory('Image', id='image')
-        self.folder.image.image_edit(file=dummy.File('foo.tiff'))
+        self.folder.image.image_edit(file=dummy.Image('foo.gif'))
 
     def testFileMimeType(self):
         self.assertEqual(self.folder.file.Format(), 'application/pdf')
-        self.assertEqual(self.folder.file.content_type, 'application/pdf')
+        self.assertEqual(self.folder.file.getFile().content_type, 'application/pdf')
 
     def testImageMimeType(self):
-        self.assertEqual(self.folder.image.Format(), 'image/tiff')
-        self.assertEqual(self.folder.image.content_type, 'image/tiff')
+        self.assertEqual(self.folder.image.Format(), 'image/gif')
+        self.assertEqual(self.folder.image.content_type, 'image/gif')
 
     def testFileEditKeepsMimeType(self):
+        self.assertEqual(self.folder.file.Format(), 'application/pdf')
+        self.assertEqual(self.folder.file.getFile().content_type, 'application/pdf')
         self.folder.file.file_edit(title='Foo')
         self.assertEqual(self.folder.file.Title(), 'Foo')
         self.assertEqual(self.folder.file.Format(), 'application/pdf')
-        self.assertEqual(self.folder.file.content_type, 'application/pdf')
+        self.assertEqual(self.folder.file.getFile().content_type, 'application/pdf')
 
     def testImageEditKeepsMimeType(self):
+        self.assertEqual(self.folder.image.Format(), 'image/gif')
+        self.assertEqual(self.folder.image.content_type, 'image/gif')
         self.folder.image.image_edit(title='Foo')
         self.assertEqual(self.folder.image.Title(), 'Foo')
-        self.assertEqual(self.folder.image.Format(), 'image/tiff')
-        self.assertEqual(self.folder.image.content_type, 'image/tiff')
+        self.assertEqual(self.folder.image.Format(), 'image/gif')
+        self.assertEqual(self.folder.image.content_type, 'image/gif')
 
     def testFileRenameKeepsMimeType(self):
-        get_transaction().commit(1) # make rename work
+        self.assertEqual(self.folder.file.Format(), 'application/pdf')
+        self.assertEqual(self.folder.file.getFile().content_type, 'application/pdf')
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.file.file_edit(id='foo')
         self.assertEqual(self.folder.foo.Format(), 'application/pdf')
-        self.assertEqual(self.folder.foo.content_type, 'application/pdf')
+        self.assertEqual(self.folder.foo.getFile().content_type, 'application/pdf')
 
     def testImageRenameKeepsMimeType(self):
-        get_transaction().commit(1) # make rename work
+        self.assertEqual(self.folder.image.Format(), 'image/gif')
+        self.assertEqual(self.folder.image.content_type, 'image/gif')
+        transaction.savepoint(optimistic=True) # make rename work
         self.folder.image.image_edit(id='foo')
-        self.assertEqual(self.folder.foo.Format(), 'image/tiff')
-        self.assertEqual(self.folder.foo.content_type, 'image/tiff')
+        self.assertEqual(self.folder.foo.Format(), 'image/gif')
+        self.assertEqual(self.folder.foo.content_type, 'image/gif')
+
+
+class TestFileURL(PloneTestCase.PloneTestCase):
+    # Tests covering http://plone.org/collector/3296
+    # file:// URLs should contain correct number of slashes
+    # NOTABUG: This is how urlparse.urlparse() works.
+
+    def testFileURLWithHost(self):
+        self.folder.invokeFactory('Link', id='link', remote_url='file://foo.com/baz.txt')
+        self.assertEqual(self.folder.link.getRemoteUrl(), 'file://foo.com/baz.txt')
+
+    def testFileURLNoHost(self):
+        self.folder.invokeFactory('Link', id='link', remote_url='file:///foo.txt')
+        self.assertEqual(self.folder.link.getRemoteUrl(), 'file:///foo.txt')
+
+    def testFileURLFourSlash(self):
+        self.folder.invokeFactory('Link', id='link', remote_url='file:////foo.com/baz.txt')
+        # See urlparse.urlparse()
+        self.assertEqual(self.folder.link.getRemoteUrl(), 'file://foo.com/baz.txt')
+
+    def testFileURLFiveSlash(self):
+        self.folder.invokeFactory('Link', id='link', remote_url='file://///foo.com/baz.txt')
+        # See urlparse.urlparse()
+        self.assertEqual(self.folder.link.getRemoteUrl(), 'file:///foo.com/baz.txt')
+
+    def testFileURLSixSlash(self):
+        self.folder.invokeFactory('Link', id='link', remote_url='file://////foo.com/baz.txt')
+        # See urlparse.urlparse()
+        self.assertEqual(self.folder.link.getRemoteUrl(), 'file:////foo.com/baz.txt')
 
 
 class TestFileExtensions(PloneTestCase.PloneTestCase):
@@ -276,133 +339,105 @@ class TestFileExtensions(PloneTestCase.PloneTestCase):
     def afterSetUp(self):
         self.folder.invokeFactory('File', id=self.file_id)
         self.folder.invokeFactory('Image', id=self.image_id)
+        transaction.savepoint(optimistic=True) # make rename work
 
     def testUploadFile(self):
-        get_transaction().commit(1) # make rename work
-        self.folder[self.file_id].file_edit(file=dummy.File('fred.gif'))
-        self.failUnless('fred.gif' in self.folder.objectIds())
+        self.folder[self.file_id].file_edit(file=dummy.File('fred.txt'))
+        self.failUnless('fred.txt' in self.folder.objectIds())
 
     def testUploadImage(self):
-        get_transaction().commit(1) # make rename work
-        self.folder[self.image_id].image_edit(file=dummy.File('fred.gif'))
+        self.folder[self.image_id].image_edit(file=dummy.Image('fred.gif'))
         self.failUnless('fred.gif' in self.folder.objectIds())
 
     def DISABLED_testFileRenameKeepsExtension(self):
-        get_transaction().commit(1) # make rename work
-        self.folder[self.file_id].file_edit(id='barney', file=dummy.File())
-        self.failUnless('barney.gif' in self.folder.objectIds())
+        # XXX Wishful thinking
+        self.folder[self.file_id].file_edit(id='barney')
+        self.failUnless('barney.txt' in self.folder.objectIds())
 
     def DISABLED_testImageRenameKeepsExtension(self):
-        get_transaction().commit(1) # make rename work
-        self.folder[self.image_id].image_edit(id='barney', file=dummy.File())
+        # XXX Wishful thinking
+        self.folder[self.image_id].image_edit(id='barney')
         self.failUnless('barney.gif' in self.folder.objectIds())
 
 
-class DummySize:
+class TestBadFileIds(PloneTestCase.PloneTestCase):
 
-    __allow_access_to_unprotected_subobjects__ = 1
-
-    def __init__(self, size=0):
-        self.set_size(size)
-
-    def get_size(self):
-        return self.size
-
-    def set_size(self, size):
-        self.size = size
-
-
-class TestGetObjSize(PloneTestCase.PloneTestCase):
+    file_id = 'File.2001-01-01.12345'
+    image_id = 'Image.2001-01-01.12345'
 
     def afterSetUp(self):
-        self.ob = DummySize()
-        self.getObjSize = self.portal.getObjSize
+        self.folder.invokeFactory('File', id=self.file_id)
+        self.folder.invokeFactory('Image', id=self.image_id)
+        transaction.savepoint(optimistic=True) # make rename work
 
-    def getSize(self):
-        return self.getObjSize(obj=self.ob)
+    def testUploadBadFile(self):
+        # http://plone.org/collector/3416
+        try:
+            self.folder[self.file_id].file_edit(file=dummy.File('fred%.txt'))
+        except CopyError:
+            # Somehow we'd get one of these *sometimes* (not consistently)
+            # when running tests... since all we're testing is that the
+            # object doesn't get renamed, this shouldn't matter
+            pass
+        self.failIf('fred%.txt' in self.folder.objectIds())
 
-    def testZeroSize(self):
-        self.ob.set_size(0)
-        size = self.getSize()
-        self.assertEquals(size, '0 kB')
+    def testUploadBadImage(self):
+        # http://plone.org/collector/3518
+        try:
+            self.folder[self.image_id].image_edit(file=dummy.File('fred%.gif'))
+        except CopyError:
+            # (ditto - see above)
+            pass
+        self.failIf('fred%.gif' in self.folder.objectIds())
 
-    def testBytesSize(self):
-        self.ob.set_size(884)
-        size = self.getSize()
-        self.assertEquals(size, '1 kB')
-
-    def testKBytesSize(self):
-        self.ob.set_size(1348)
-        size = self.getSize()
-        self.assertEquals(size, '1.3 kB')
-
-    def testMBytesSize(self):
-        self.ob.set_size(1024*1024+1024*687)
-        size = self.getSize()
-        self.assertEquals(size, '1.7 MB')
-
-    def testGBytesSize(self):
-        self.ob.set_size(1024*1024*1024+1024*1024*107)
-        size = self.getSize()
-        self.assertEquals(size, '1.1 GB')
+    # TODO: Dang! No easy way to get at the validator state...
 
 
-class DummyDefaultPage:
+class TestImagePatch(PloneTestCase.PloneTestCase):
 
-    __allow_access_to_unprotected_subobjects__ = 1
+    def testImagePatchComputedProps(self):
+        from Products.CMFPlone.patches.imagePatch import tag
+        kw = {'_title':'some title',
+              '_alt':'alt tag',
+              'height':100,
+              'width':100}
+        # Wrap object so that ComputedAttribute gets executed.
+        self.ob = dummy.ImageComputedProps(**kw).__of__(self.folder)
 
-    def __init__(self, default_page=['test'], keys=['index_html']):
-        self.keys = keys
-        self.set_default(default_page)
-
-    def set_default(self, default, has_key=1):
-        self.default_page = default
-        self.hk = has_key
-
-    def has_key(self, key):
-        return self.hk or key in self.keys
+        endswith = ('alt="alt tag" title="some title" '
+                    'height="100" width="100" />')
+        self.assertEqual(tag(self.ob)[-len(endswith):], endswith)
 
 
-class TestDefaultPage(PloneTestCase.PloneTestCase):
+class TestWorkflowHistory(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
-        self.ob = DummyDefaultPage()
-        self.util = self.portal.plone_utils
-        sp = self.portal.portal_properties.site_properties
-        self.default = sp.getProperty('default_page', [])
+        self.folder.invokeFactory('Document',id='doc')
+        self.doc = self.folder.doc
+        self.wf = self.portal.portal_workflow
 
-    def getDefault(self):
-        return self.util.browserDefault(self.ob)
+    def test_WorkflowHistory(self):
+        # see if doc returns any history at this point
+        history = self.doc.getWorkflowHistory()
+        self.failUnlessEqual(history, []) 
 
-    def testDefaultPageSetting(self):
-        self.assertEquals(self.default, ('index_html', 'index.html',
-                                         'index.htm', 'FrontPage'))
+        # do a transition 
+        self.setRoles(['Reviewer']) # has no modify portal content permission
+        self.wf.doActionFor(self.doc, 'publish')
+        self.wf.doActionFor(self.doc, 'retract')
+        self.wf.doActionFor(self.doc, 'publish')
+        history = self.doc.getWorkflowHistory()
 
-    def testDefaultPageStringExist(self):
-        # Test for https://plone.org/collector/2671
-        self.ob.set_default('test')
-        self.assertEquals(self.getDefault(), (self.ob, ['test']))
+        # should return no history: no edit permission
+        self.failUnlessEqual([],[])
 
-    def testDefaultPageStringNotExist(self):
-        # Test for https://plone.org/collector/2671
-        self.ob.set_default('test', 0)
-        self.assertEquals(self.getDefault(), (self.ob, ['index_html']))
-
-    def testDefaultPageSequenceExist(self):
-        # Test for https://plone.org/collector/2671
-        self.ob.set_default(['test'])
-        self.assertEquals(self.getDefault(), (self.ob, ['test']))
-
-    def testDefaultPageSequenceNotExist(self):
-        # Test for https://plone.org/collector/2671
-        self.ob.set_default(['test'], 0)
-        self.assertEquals(self.getDefault(), (self.ob, ['index_html']))
-        self.ob.keys = ['index.html']
-        self.assertEquals(self.getDefault(), (self.ob, ['index.html']))
-        self.ob.keys = ['index.htm']
-        self.assertEquals(self.getDefault(), (self.ob, ['index.htm']))
-        self.ob.keys = ['FrontPage']
-        self.assertEquals(self.getDefault(), (self.ob, ['FrontPage']))
+        # give edit permissions.
+        self.doc.manage_permission('Modify portal content', ['Reviewer'], acquire=0)
+        # should return the history list
+        history = self.doc.getWorkflowHistory()
+        self.failUnlessEqual(history[0]['action'], 'publish')
+        self.failUnlessEqual(history[1]['action'], 'retract')
+        self.failUnlessEqual(history[2]['action'], 'publish')
 
 
 def test_suite():
@@ -411,9 +446,11 @@ def test_suite():
     suite.addTest(makeSuite(TestContentTypeScripts))
     suite.addTest(makeSuite(TestEditShortName))
     suite.addTest(makeSuite(TestEditFileKeepsMimeType))
+    suite.addTest(makeSuite(TestFileURL))
     suite.addTest(makeSuite(TestFileExtensions))
-    suite.addTest(makeSuite(TestGetObjSize))
-    suite.addTest(makeSuite(TestDefaultPage))
+    suite.addTest(makeSuite(TestBadFileIds))
+    suite.addTest(makeSuite(TestImagePatch))
+    suite.addTest(makeSuite(TestWorkflowHistory))
     return suite
 
 if __name__ == '__main__':
