@@ -12,10 +12,10 @@ from Products.CMFPlone.tests import PloneTestCase
 from Products.CMFCore.WorkflowCore import WorkflowException
 
 from Products.CMFCore.utils import _checkPermission as checkPerm
-from Products.CMFCore.CMFCorePermissions import AccessContentsInformation
-from Products.CMFCore.CMFCorePermissions import View
-from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
-from Products.CMFCalendar.EventPermissions import ChangeEvents
+from Products.CMFCore.permissions import AccessContentsInformation
+from Products.CMFCore.permissions import View
+from Products.CMFCore.permissions import ModifyPortalContent
+from Products.CMFCalendar.permissions import ChangeEvents
 
 default_user = PloneTestCase.default_user
 
@@ -170,13 +170,13 @@ class TestDefaultWorkflow(PloneTestCase.PloneTestCase):
         self.workflow.doActionFor(self.doc, 'submit')
         # Owner is allowed
         self.failUnless(checkPerm(View, self.doc))
-        # Member is allowed (XXX:?)
+        # Member is allowed (TODO:?)
         self.login('member')
         self.failUnless(checkPerm(View, self.doc))
         # Reviewer is allowed
         self.login('reviewer')
         self.failUnless(checkPerm(View, self.doc))
-        # Anonymous is allowed (XXX:?)
+        # Anonymous is allowed (TODO:?)
         self.logout()
         self.failUnless(checkPerm(View, self.doc))
 
@@ -245,13 +245,13 @@ class TestDefaultWorkflow(PloneTestCase.PloneTestCase):
         self.workflow.doActionFor(self.doc, 'submit')
         # Owner is allowed
         self.failUnless(checkPerm(AccessContentsInformation, self.doc))
-        # Member is allowed (XXX:?)
+        # Member is allowed (TODO:?)
         self.login('member')
         self.failUnless(checkPerm(AccessContentsInformation, self.doc))
         # Reviewer is allowed
         self.login('reviewer')
         self.failUnless(checkPerm(AccessContentsInformation, self.doc))
-        # Anonymous is allowed (XXX:?)
+        # Anonymous is allowed (TODO:?)
         self.logout()
         self.failUnless(checkPerm(AccessContentsInformation, self.doc))
 
@@ -463,13 +463,13 @@ class TestDefaultWorkflow(PloneTestCase.PloneTestCase):
         self.workflow.doActionFor(self.doc, 'submit')
         # Owner is allowed
         self.failUnless(self.catalog(id='doc'))
-        # Member is allowed (XXX:?)
+        # Member is allowed (TODO:?)
         self.login('member')
         self.failUnless(self.catalog(id='doc'))
         # Reviewer is allowed
         self.login('reviewer')
         self.failUnless(self.catalog(id='doc'))
-        # Anonymous is allowed (XXX:?)
+        # Anonymous is allowed (TODO:?)
         self.logout()
         self.failUnless(self.catalog(id='doc'))
 
@@ -503,6 +503,18 @@ class TestDefaultWorkflow(PloneTestCase.PloneTestCase):
         self.failUnless(len(worklist) == 1)
         self.failUnless(worklist[0] == self.doc)
 
+    def testStateTitles(self):
+        state_titles = { 'private': 'Private',
+                        'visible': 'Public Draft',
+                        'pending': 'Pending',
+                        'published': 'Published'
+                        }
+        for wf in self.workflow.objectValues():
+            for state_id, title in state_titles.items():
+                state = getattr(wf.states, state_id, None)
+                if state is not None:
+                    self.assertEqual(state.title, title)
+
 
 def test_suite():
     from unittest import TestSuite, makeSuite
