@@ -16,9 +16,8 @@ class TestMigrationTool(PloneTestCase.PloneTestCase):
         self.migration = self.portal.portal_migration
 
     def testMigrationFinished(self):
-        # If om_icons() returns more than 1 icon 
-        # migration has failed.
-        self.assertEqual(len(self.migration.om_icons()), 1,
+        self.assertEqual(self.migration.getInstanceVersion(),
+                         self.migration.getFileSystemVersion(),
                          'Migration failed')
 
     def testMigrationNeedsUpgrading(self):
@@ -33,6 +32,16 @@ class TestMigrationTool(PloneTestCase.PloneTestCase):
         self.failIf(self.migration.needRecatalog(),
                     'Migration needs recataloging')
 
+    def testForceMigration(self):
+        self.setRoles(['Manager'])
+        # Make sure we don't embarrass ourselves again...
+        version = '2.0.5'
+        while version is not None:
+            version, msg = self.migration._upgrade(version)
+        expect = 'Migration completed at version %s' % \
+                 self.migration.getFileSystemVersion()
+        self.assertEqual(msg[0], expect)
+
 
 def test_suite():
     from unittest import TestSuite, makeSuite
