@@ -8,6 +8,7 @@ if __name__ == '__main__':
 
 from Testing import ZopeTestCase
 from Products.CMFPlone.tests import PloneTestCase
+from Products.CMFCore.permissions import AddPortalContent
 
 from AccessControl import Unauthorized
 default_user = PloneTestCase.default_user
@@ -17,38 +18,37 @@ def sortTuple(t):
     l.sort()
     return tuple(l)
 
+ADD_DOC_PERM = 'ATContentTypes: Add Document'
+
 
 class TestPortalFactory(PloneTestCase.PloneTestCase):
 
     def afterSetUp(self):
         self.membership = self.portal.portal_membership
-        self.portal.acl_users._doAddUser('member', 'secret', ['Member'], [])
-        self.portal.acl_users._doAddUser('reviewer', 'secret', ['Reviewer'], [])
-        self.portal.acl_users._doAddUser('manager', 'secret', ['Manager'], [])
-        
+        self.membership.addMember('member', 'secret', ['Member'], [])
+        self.membership.addMember('manager', 'secret', ['Manager'], [])
+
     def testTraverse(self):
         temp_doc = self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
-        self.assertEqual(temp_doc.meta_type, 'Document')
+        self.assertEqual(temp_doc.portal_type, 'Document')
         self.assertEqual(temp_doc.getId(), 'tmp_id')
 
     def testTraverseTwiceByDifferentContentTypes(self):
         temp_doc = self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
-        self.assertEqual(temp_doc.meta_type, 'Document')
+        self.assertEqual(temp_doc.portal_type, 'Document')
         self.assertEqual(temp_doc.getId(), 'tmp_id')
         temp_img = self.folder.restrictedTraverse('portal_factory/Image/tmp_id_image')
-        self.assertEqual(temp_img.meta_type, 'Portal Image')
+        self.assertEqual(temp_img.portal_type, 'Image')
         self.assertEqual(temp_img.getId(), 'tmp_id_image')
 
     def testTempFolderLocalRoles(self):
-        self.membership.addMember('user2', 'secret', ['Member'], [])
+        # Temporary objects should "inherit" local roles from container
+        member = self.membership.getMemberById('member')
         self.portal._addRole('Foo')
 
-        member = self.membership.getMemberById('user2')
-        user = member.getUser()
-
-        self.folder.manage_addLocalRoles('user2', ('Foo',))
-        self.folder.invokeFactory(id='folder2', type_name='Folder')
-        self.folder.folder2.manage_addLocalRoles('user2', ('Reviewer',))
+        self.folder.manage_addLocalRoles('member', ('Foo',))
+        self.folder.invokeFactory('Folder', id='folder2')
+        self.folder.folder2.manage_addLocalRoles('member', ('Reviewer',))
 
         self.assertEqual(sortTuple(member.getRolesInContext(self.folder)),
                          ('Authenticated', 'Foo', 'Member'))
@@ -60,28 +60,134 @@ class TestPortalFactory(PloneTestCase.PloneTestCase):
         temp_object2 = self.folder.folder2.restrictedTraverse('portal_factory/Document/tmp_id')
         self.assertEqual(sortTuple(member.getRolesInContext(temp_object2)),
                          ('Authenticated', 'Foo', 'Member', 'Reviewer'))
+        
+    def testTempFolderLocalRolesWithBlocking(self):
+        # Temporary objects should "inherit" local roles from container,
+        # but also need to respect PLIP 16 local role blocking
+        member = self.membership.getMemberById('member')
+        self.portal._addRole('Foo')
 
-    def testTempObjectLocalRoles(self):
+        self.folder.manage_addLocalRoles('member', ('Foo',))
+        self.folder.invokeFactory('Folder', id='folder2')
+        self.folder.folder2.manage_addLocalRoles('member', ('Reviewer',))
+        # make folder2 not inherit local roles
+        self.portal.plone_utils.acquireLocalRoles(self.folder.folder2, status=0)
+
+        self.assertEqual(sortTuple(member.getRolesInContext(self.folder.folder2)),
+                         ('Authenticated', 'Member', 'Reviewer'))
+
+        temp_object2 = self.folder.folder2.restrictedTraverse('portal_factory/Document/tmp_id')
+        self.assertEqual(sortTuple(member.getRolesInContext(temp_object2)),
+                         ('Authenticated', 'Member', 'Reviewer'))
+
+    def testTempFolderPermissionAq(self):
+        from Products.CMFPlone.FactoryTool import FACTORY_INFO
+        # Temporary folder should acquire same permissions as the intended parent
+        self.login('manager')
+        self.portal.invokeFactory('Folder', id='folder1')
+        folder1 = self.portal.folder1
+        folder1.invokeFactory('Folder', id='folder2')
+        folder2 = folder1.folder2
+        folder2.invokeFactory('Folder', id='folder3')
+        folder3 = folder1.folder2.folder3
+        folder3.restrictedTraverse('portal_factory/Document/tmp_id')
+        # clear out the cached factory info (this would happen by itself in an actual request)
+        self.portal.REQUEST.set(FACTORY_INFO, {})
+
+        # make sure no permission is available initially
+        self.login('member')
+        pm = self.portal.portal_membership
+        assert (not pm.checkPermission(ADD_DOC_PERM, folder3))
+        self.assertRaises(Unauthorized, folder3.restrictedTraverse, 'portal_factory/Document/tmp_id/atct_edit')
+        # clear out the cached factory info (this would happen by itself in an actual request)
+        self.portal.REQUEST.set(FACTORY_INFO, {})
+
+        # grant the add permission on the parent
+        self.login('manager')
+        folder3.manage_permission(ADD_DOC_PERM, ['Authenticated'], 1)
+
+        # now make sure we can see it
+        self.login('member')
+        assert (pm.checkPermission(ADD_DOC_PERM, folder3))
+        #folder3.setConstrainTypesMode(0)
+        folder3.restrictedTraverse('portal_factory/Document/tmp_id/atct_edit')
+        # clear out the cached factory info (this would happen by itself in an actual request)
+        self.portal.REQUEST.set(FACTORY_INFO, {})
+
+        # blow away folder3 and start again
+        self.login('manager')
+        folder2.manage_delObjects(['folder3'])
+        folder2.invokeFactory('Folder', id='folder3')
+        folder3 = folder1.folder2.folder3
+
+        # grant the add permission on the grandparent
+        folder2.manage_permission(ADD_DOC_PERM, ['Authenticated'], 1)
+
+        # now make sure we can see it
+        self.login('member')
+        assert (pm.checkPermission(ADD_DOC_PERM, folder3))
+        folder3.restrictedTraverse('portal_factory/Document/tmp_id')
+        # clear out the cached factory info (this would happen by itself in an actual request)
+        self.portal.REQUEST.set(FACTORY_INFO, {})
+
+        # blow away folder2 and start again
+        self.login('manager')
+        folder1.manage_delObjects(['folder2'])
+        folder1.invokeFactory('Folder', id='folder2')
+        folder2 = folder1.folder2
+        folder2.invokeFactory('Folder', id='folder3')
+        folder3 = folder1.folder2.folder3
+
+        # add the permission on the portal root
+        self.portal.manage_permission(ADD_DOC_PERM, ['Authenticated'], 1)
+
+        # now make sure we can see it
+        self.login('member')
+        assert (pm.checkPermission(ADD_DOC_PERM, folder3))
+        folder3.restrictedTraverse('portal_factory/Document/tmp_id')
+        # clear out the cached factory info (this would happen by itself in an actual request)
+        self.portal.REQUEST.set(FACTORY_INFO, {})
+
+
+    def testTempObjectLocalRolesBug(self):
+        # Evil monkey patch should not change all objects of a class
+        self.createMemberarea('member')
         member = self.membership.getMemberById('member')
 
+        # Make an unrelated non-temporary object for comparison
         self.login('manager')
-        self.portal.invokeFactory(id='nontmp_id', type_name='Document')
+        self.portal.invokeFactory('Document', id='nontmp_id')
         nontemp_object = getattr(self.portal, 'nontmp_id')
-        
-        # assume identify of the ordinary member
+
+        # Assume identify of the ordinary member
         self.login('member')
-        folder = self.portal.Members.member
+        folder = self.membership.getHomeFolder()
         temp_object = folder.restrictedTraverse('portal_factory/Document/tmp_id')
-        # make sure user is owner of temporary object
+
+        # Make sure member is owner of temporary object
         self.assertEqual(sortTuple(member.getRolesInContext(temp_object)),
                          ('Authenticated', 'Member', 'Owner'))
         self.assertEqual(temp_object.Creator(), 'member')
-        # make sure user is not owner of non-temporary object 
-        # (i.e. make sure our evil monkey patch of the temporary instance has 
+
+        # Make sure member is not owner of non-temporary object
+        # (i.e. make sure our evil monkey patch of the temporary instance has
         # not resulted in our patching all instances of the class)
         self.assertEqual(sortTuple(member.getRolesInContext(nontemp_object)),
                          ('Authenticated', 'Member'))
 
+    def testTempFolderPermissions(self):
+        # TempFolder should "inherit" permission mappings from container
+        previous_roles = [r for r in self.folder.rolesOfPermission(AddPortalContent) if r['name'] == 'Anonymous']
+        self.folder.manage_permission(AddPortalContent, ['Anonymous'], 1)
+        new_roles = [r for r in self.folder.rolesOfPermission(AddPortalContent) if r['name'] == 'Anonymous']
+        self.failIfEqual(previous_roles, new_roles)
+
+        temp_folder = self.folder.restrictedTraverse(
+                                'portal_factory/Document/tmp_id').aq_parent
+        temp_roles = [r for r in temp_folder.rolesOfPermission(AddPortalContent) if r['name'] == 'Anonymous']
+        
+        self.assertEqual(temp_roles, new_roles)
+
 
 class TestCreateObject(PloneTestCase.PloneTestCase):
 
@@ -96,7 +202,6 @@ class TestCreateObject(PloneTestCase.PloneTestCase):
         # Anonymous should not be able to create the (real) object
         # Note that Anonymous used to be able to create the temp object...
         temp_object = self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
-        # but not the final one
         self.logout()
         self.assertRaises(Unauthorized, temp_object.portal_factory.doCreate,
                           temp_object, 'foo')
@@ -110,11 +215,10 @@ class TestCreateObject(PloneTestCase.PloneTestCase):
         self.assertEqual(self.folder.foo.get_local_roles_for_userid(default_user), ('Owner',))
 
     def testUnauthorizedToCreateObjectByDocumentEdit(self):
-        # Anonymous should not be edit the (real) object
+        # Anonymous should not be able to create the (real) object
         # Note that Anonymous used to be able to create the temp object...
         temp_object = self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
         self.logout()
-        # but not the final one
         self.assertRaises(Unauthorized, temp_object.document_edit,
                           id='foo', title='Foo', text_format='plain', text='')
 
@@ -134,19 +238,19 @@ class TestCreateObjectByURL(PloneTestCase.FunctionalTestCase):
 
     def testCreateObject(self):
         # createObject script should make a temp object
-        response = self.publish(self.folder_path + 
+        response = self.publish(self.folder_path +
                                 '/createObject?type_name=Document',
                                 self.basic_auth)
 
         self.assertEqual(response.getStatus(), 302) # Redirect to document_edit_form
 
         # The redirect URL should contain the factory parts
-        body = response.getBody()
-        self.failUnless(body.startswith(self.folder_url+'/portal_factory/Document/'))
-        self.failUnless(body.endswith('/document_edit_form'))
+        location = response.getHeader('Location')
+        self.failUnless(location.startswith(self.folder_url+'/portal_factory/Document/'))
+        self.failUnless(location.endswith('/atct_edit'))
 
         # Perform the redirect
-        edit_form_path = body[len(self.app.REQUEST.SERVER_URL):]
+        edit_form_path = location[len(self.app.REQUEST.SERVER_URL):]
         response = self.publish(edit_form_path, self.basic_auth)
         self.assertEqual(response.getStatus(), 200) # OK
 
@@ -160,12 +264,12 @@ class TestCreateObjectByURL(PloneTestCase.FunctionalTestCase):
         self.assertEqual(response.getStatus(), 302) # Redirect to document_edit_form
 
         # The redirect URL should contain the factory parts
-        body = response.getBody()
-        self.failUnless(body.startswith(self.folder_url+'/portal_factory/Document/'))
-        self.failUnless(body.endswith('/document_edit_form'))
+        location = response.getHeader('Location')
+        self.failUnless(location.startswith(self.folder_url+'/portal_factory/Document/'))
+        self.failUnless(location.endswith('/atct_edit'))
 
         # Perform the redirect
-        edit_form_path = body[len(self.app.REQUEST.SERVER_URL):]
+        edit_form_path = location[len(self.app.REQUEST.SERVER_URL):]
         response = self.publish(edit_form_path, self.basic_auth)
         self.assertEqual(response.getStatus(), 200) # OK
 
@@ -179,11 +283,10 @@ class TestCreateObjectByURL(PloneTestCase.FunctionalTestCase):
 
     def testUnauthorizedToViewEditFormOfNonFactoryObject(self):
         # Anonymous should not be able to see newsitem_edit_form
-        response = self.publish(self.folder_path + 
+        response = self.publish(self.folder_path +
                                 '/createObject?type_name=News%20Item',
                                 ) # No basic out info
 
-        # Ok, so here we don't even get far enough for the redirect to occur
         self.assertEqual(response.getStatus(), 401) # Unauthorized
 
     def testCreateObjectByDocumentEdit(self):
@@ -193,7 +296,10 @@ class TestCreateObjectByURL(PloneTestCase.FunctionalTestCase):
             self.basic_auth)
 
         self.assertEqual(response.getStatus(), 302) # Redirect to document_view
-        self.failUnless(response.getBody().startswith(self.folder_url+'/foo/document_view'))
+        viewAction = self.portal.portal_types['Document'].getActionById('view')
+        if viewAction and not viewAction[0] == '/':
+            viewAction = "/" + viewAction
+        self.failUnless(response.getHeader('Location').startswith(self.folder_url+'/foo' + viewAction))
 
         self.failUnless('foo' in self.folder.objectIds())
         self.assertEqual(self.folder.foo.Title(), 'Foo')
