@@ -9,9 +9,8 @@ if __name__ == '__main__':
 from Testing import ZopeTestCase
 from Products.CMFPlone.tests import PloneTestCase
 
-from Acquisition import aq_base
 from AccessControl import Unauthorized
-from Products.CMFCore.CMFCorePermissions import AddPortalMember
+from Products.CMFCore.permissions import AddPortalMember
 
 member_id = 'new_member'
 
@@ -75,7 +74,13 @@ class TestRegistrationTool(PloneTestCase.PloneTestCase):
         # http://plone.org/collector/3000
         self.portal.manage_permission(AddPortalMember, ['Manager'], acquire=0)
         self.app.REQUEST['username'] = member_id
+        # TODO: register has a proxy role but we trip over
+        # validate_registration... (2.0.5)
         self.assertRaises(Unauthorized, self.portal.register)
+
+    def testInvalidCMFDefaultEmailFailsValidation(self):
+        # http://plone.org/collector/3910
+        self.failIf(self.registration.isValidEmail('bogus@127.0.0.1'))
     
 
 class TestPasswordGeneration(PloneTestCase.PloneTestCase):
