@@ -3,25 +3,26 @@
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """ Programmatically creates a workflow.
 
-$Id: Default.py,v 1.14 2003/09/01 15:18:36 yuppie Exp $
+$Id: Default.py 36457 2004-08-12 15:07:44Z jens $
 """
 
-from Products.CMFCore.CMFCorePermissions import AccessContentsInformation
-from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
-from Products.CMFCore.CMFCorePermissions import RequestReview
-from Products.CMFCore.CMFCorePermissions import ReviewPortalContent
-from Products.CMFCore.CMFCorePermissions import View
 from Products.CMFCore.WorkflowTool import addWorkflowFactory
-from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
+
+from DCWorkflow import DCWorkflowDefinition
+from permissions import AccessContentsInformation
+from permissions import ModifyPortalContent
+from permissions import RequestReview
+from permissions import ReviewPortalContent
+from permissions import View
 
 p_access = AccessContentsInformation
 p_modify = ModifyPortalContent
@@ -226,8 +227,8 @@ def setupDefaultWorkflowClassic(wf):
     sdef.setProperties(
         title='Waiting for reviewer',
         transitions=('publish', 'reject', 'retract',))
-    sdef.setPermission(p_access, 1, (r_manager, r_owner, r_reviewer))
-    sdef.setPermission(p_view, 1, (r_manager, r_owner, r_reviewer))
+    sdef.setPermission(p_access, 0, (r_manager, r_owner, r_reviewer))
+    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_reviewer))
     sdef.setPermission(p_modify, 0, (r_manager, r_reviewer))
 
     sdef = wf.states['published']
