@@ -1,30 +1,30 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
-# 
+# FOR A PARTICULAR PURPOSE.
+#
 ##############################################################################
 """ Scripts in a web-configurable workflow.
 
-$Id: Scripts.py,v 1.3 2002/08/01 19:05:14 tseaver Exp $
+$Id: Scripts.py 37045 2005-06-14 18:09:32Z sidnei $
 """
 
 from OFS.Folder import Folder
-import Globals
+from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 
-from Products.CMFCore.CMFCorePermissions import ManagePortal
-
 from ContainerTab import ContainerTab
+from permissions import ManagePortal
 
 
 class Scripts (ContainerTab):
+    """A container for workflow scripts"""
 
     meta_type = 'Workflow Scripts'
 
@@ -35,6 +35,7 @@ class Scripts (ContainerTab):
         '''
         '''
         kw['management_view'] = 'Scripts'
-        return apply(Folder.manage_main, (self, client, REQUEST), kw)
+        m = Folder.manage_main.__of__(self)
+        return m(self, client, REQUEST, **kw)
 
-Globals.InitializeClass(Scripts)
+InitializeClass(Scripts)
