@@ -1,24 +1,50 @@
-##############################################################################
-#
-# Copyright (c) 2003 Connexions Projects and Contributors. All Rights Reserved.
-#
-##############################################################################
-""" Basic usergroup tool.
+# -*- coding: utf-8 -*-
+## GroupUserFolder
+## Copyright (C)2006 Ingeniweb
 
-$Id: GroupsTool.py,v 1.23 2004/05/04 21:48:46 dreamcatcher Exp $
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+
+## You should have received a copy of the GNU General Public License
+## along with this program; see the file COPYING. If not, write to the
+## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+## Copyright (c) 2003 The Connexions Project, All Rights Reserved
+## initially written by J Cameron Cooper, 11 June 2003
+## concept with Brent Hendricks, George Runyan
 """
+Basic usergroup tool.
+"""
+__version__ = "$Revision:  $"
+# $Source:  $
+# $Id: GroupsTool.py 30098 2006-09-08 12:35:01Z encolpe $
+__docformat__ = 'restructuredtext'
 
 from Products.CMFCore.utils import UniqueObject
 from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.utils import _checkPermission
 from OFS.SimpleItem import SimpleItem
 from Globals import InitializeClass, DTMLFile, MessageDialog
 from Acquisition import aq_base
 from AccessControl.User import nobody
 from AccessControl import ClassSecurityInfo
-from Products.CMFCore.CMFCorePermissions import View
-from Products.CMFCore.CMFCorePermissions import AccessContentsInformation
-from Products.CMFCore.CMFCorePermissions import ManagePortal
-from Products.CMFCore.CMFCorePermissions import ViewManagementScreens
+# BBB CMF < 1.5
+try:
+    from Products.CMFCore.permissions import ManagePortal
+    from Products.CMFCore.permissions import View
+    from Products.CMFCore.permissions import ViewManagementScreens
+except ImportError:
+    from Products.CMFCore.CMFCorePermissions import ManagePortal
+    from Products.CMFCore.CMFCorePermissions import View
+    from Products.CMFCore.CMFCorePermissions import ViewManagementScreens
+
 from GroupsToolPermissions import AddGroups
 from GroupsToolPermissions import ManageGroups
 from GroupsToolPermissions import DeleteGroups
@@ -26,13 +52,13 @@ from GroupsToolPermissions import ViewGroups
 from GroupsToolPermissions import SetGroupOwnership
 from Products.CMFCore.ActionProviderBase import ActionProviderBase
 from interfaces.portal_groups import portal_groups as IGroupsTool
+from global_symbols import *
 
-#import zLOG
+# Optional feature-preview support
+import PloneFeaturePreview
 
-#def log(message,summary='',severity=0):
-#    zLOG.LOG('GroupUserFolder: ',severity,summary,message)
 
-class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
+class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase, ):
     """ This tool accesses group data through a GRUF acl_users object.
 
     It can be replaced with something that groups member data in a
@@ -65,9 +91,9 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                      },
                 ) + SimpleItem.manage_options)
 
-    #
-    #   ZMI methods
-    #
+    #                                                   #
+    #                   ZMI methods                     #
+    #                                                   #
     security.declareProtected(ViewManagementScreens, 'manage_overview')
     manage_overview = DTMLFile('dtml/explainGroupsTool', globals())     # unlike MembershipTool
     security.declareProtected(ViewManagementScreens, 'manage_config')
@@ -93,19 +119,19 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
 
     security.declareProtected(ViewGroups, 'getGroupById')
     def getGroupById(self, id):
-        """Returns the portal_groupdata-ish object for a group corresponding
-        to this id."""
+        """
+        Returns the portal_groupdata-ish object for a group corresponding to this id.
+        """
         if id==None:
             return None
-        prefix = self.acl_users.getGroupPrefix()
-        g = self.acl_users.getGroup(id, prefixed=id.startswith(prefix))
+        g = self.acl_users.getGroupByName(id, None)
         if g is not None:
             g = self.wrapGroup(g)
         return g
 
     security.declareProtected(ViewGroups, 'getGroupsByUserId')
     def getGroupsByUserId(self, userid):
-        """Returns a list of the groups the user corresponding to 'userid' belongs to."""
+        """Return a list of the groups the user corresponding to 'userid' belongs to."""
         #log("getGroupsByUserId(%s)" % userid)
         user = self.acl_users.getUser(userid)
         #log("user '%s' is in groups %s" % (userid, user.getGroups()))
@@ -117,26 +143,31 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
 
     security.declareProtected(ViewGroups, 'listGroups')
     def listGroups(self):
-        """Returns a list of the available portal_groupdata-ish objects."""
-        return [self.wrapGroup(elt) for elt in self.acl_users.getGroups()]
+        """Return a list of the available portal_groupdata-ish objects."""
+        return [ self.wrapGroup(elt) for elt in self.acl_users.getGroups() ]
 
     security.declareProtected(ViewGroups, 'listGroupIds')
     def listGroupIds(self):
-        """Returns a list of the available groups' ids as entered (without group prefixes)."""
-        return self.acl_users.getGroupNames(prefixed=0)
+        """Return a list of the available groups' ids as entered (without group prefixes)."""
+        return self.acl_users.getGroupNames()
 
-    security.declarePrivate('getPureUserNames')
-    def getPureUserNames(self):
-        """Get the usernames (ids) of only users. """
-        return self.acl_users.getPureUserNames()
+    security.declareProtected(ViewGroups, 'listGroupNames')
+    def listGroupNames(self):
+        """Return a list of the available groups' ids as entered (without group prefixes)."""
+        return self.acl_users.getGroupNames()
 
-    security.declarePrivate('getPureUsers')
-    def getPureUsers(self):
-        """Get the actual (unwrapped) user objects of only users. """
-        return self.acl_users.getPureUsers()
+    security.declarePublic("isGroup")
+    def isGroup(self, u):
+        """Test if a user/group object is a group or not.
+        You must pass an object you get earlier with wrapUser() or wrapGroup()
+        """
+        base = aq_base(u)
+        if hasattr(base, "isGroup") and base.isGroup():
+            return 1
+        return 0
 
     security.declareProtected(View, 'searchForGroups')
-    def searchForGroups(self, REQUEST, **kw):
+    def searchForGroups(self, REQUEST = {}, **kw):
         """Return a list of groups meeting certain conditions. """
         # arguments need to be better refined?
         if REQUEST:
@@ -148,6 +179,8 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         email = dict.get('email', None)
         roles = dict.get('roles', None)
         title = dict.get('title', None)
+        title_or_name = dict.get('title_or_name', None)
+        
         last_login_time = dict.get('last_login_time', None)
         #is_manager = self.checkPermission('Manage portal', self)
 
@@ -161,6 +194,8 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
             email = None
         if title:
             title = title.strip().lower()
+        if title_or_name:
+            title_or_name = title_or_name.strip().lower()
         if not title:
             title = None
 
@@ -187,6 +222,13 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
             if title:
                 if g.title.lower().find(title) == -1:
                     continue
+            if title_or_name:
+                # first search for title
+                if g.title.lower().find(title_or_name) == -1:
+                    # not found, now search for name
+                    if (g.getGroupName().lower().find(title_or_name) == -1) and (g.getGroupId().lower().find(title_or_name) == -1):
+                        continue
+                
             if last_login_time:
                 if g.last_login_time < last_login_time:
                     continue
@@ -195,21 +237,25 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         return res
 
     security.declareProtected(AddGroups, 'addGroup')
-    def addGroup(self, id, password = None, roles = [], domains = ()):
+    def addGroup(self, id, roles = [], groups = [], *args, **kw):
         """Create a group, and a group workspace if the toggle is on, with the supplied id, roles, and domains.
 
         Underlying user folder must support adding users via the usual Zope API.
         Passwords for groups ARE irrelevant in GRUF."""
-        self.acl_users.Groups.acl_users.userFolderAddUser(id, password, roles, domains)
+        if id in self.listGroupIds():
+            raise ValueError, "Group '%s' already exists." % (id, )
+        self.acl_users.userFolderAddGroup(id, roles = roles, groups = groups )
         self.createGrouparea(id)
+        self.getGroupById(id).setProperties(**kw)
 
     security.declareProtected(ManageGroups, 'editGroup')
-    def editGroup(self, id, password, roles, permissions):
+    def editGroup(self, id, roles = None, groups = None, *args, **kw):
         """Edit the given group with the supplied password, roles, and domains.
 
         Underlying user folder must support editing users via the usual Zope API.
         Passwords for groups seem to be currently irrelevant in GRUF."""
-        self.acl_users.Groups.acl_users.userFolderEditUser(id, password, roles, permissions)
+        self.acl_users.userFolderEditGroup(id, roles = roles, groups = groups, )
+        self.getGroupById(id).setProperties(**kw)
 
     security.declareProtected(DeleteGroups, 'removeGroups')
     def removeGroups(self, ids, keep_workspaces=0):
@@ -224,7 +270,7 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
             for guser in gusers:
                 gdata.removeMember(guser.id)
 
-        self.acl_users.Groups.acl_users.userFolderDelUsers(ids)
+        self.acl_users.userFolderDelGroups(ids)
         gwf = self.getGroupWorkspacesFolder()
         if not gwf: # _robert_
             return
@@ -239,12 +285,10 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
 
         For GRUF this is easy. Others may have to re-implement."""
         user = group.getGroup()
-        if user is not None:
-            object.changeOwnership(user)
-            object.manage_setLocalRoles(user.getId(), ['Owner'])
-        else:
-            pass
-                # should we have an error here?
+        if user is None:
+            raise ValueError, "Invalid group: '%s'." % (group, )
+        object.changeOwnership(user)
+        object.manage_setLocalRoles(user.getId(), ['Owner'])
 
     security.declareProtected(ManagePortal, 'setGroupWorkspacesFolder')
     def setGroupWorkspacesFolder(self, id="", title=""):
@@ -312,8 +356,11 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         if id and self.getGroupWorkspacesCreationFlag():
             if workspaces is None:
                 # add GroupWorkspaces folder
-
-                parent.invokeFactory(self.getGroupWorkspaceContainerType(), self.getGroupWorkspacesFolderId())
+                pt.constructContent(
+                    type_name = self.getGroupWorkspaceContainerType(),
+                    container = parent,
+                    id = self.getGroupWorkspacesFolderId(),
+                    )
                 workspaces = self.getGroupWorkspacesFolder()
                 workspaces.setTitle(self.getGroupWorkspacesFolderTitle())
                 workspaces.setDescription("Container for " + self.getGroupWorkspacesFolderId())
@@ -323,10 +370,14 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                 portal_catalog = getToolByName( self, 'portal_catalog' )
                 portal_catalog.unindexObject(workspaces)     # unindex GroupWorkspaces folder
                 workspaces._setProperty('right_slots', (), 'lines')
-
-            if workspaces is not None and not hasattr(workspaces, id):
+                
+            if workspaces is not None and not hasattr(workspaces.aq_base, id):
                 # add workspace to GroupWorkspaces folder
-                workspaces.invokeFactory(self.getGroupWorkspaceType(), id)
+                pt.constructContent(
+                    type_name = self.getGroupWorkspaceType(),
+                    container = workspaces,
+                    id = id,
+                    )
                 space = self.getGroupareaFolder(id)
                 space.setTitle("%s workspace" % id)
                 space.setDescription("Container for objects shared by this group")
@@ -340,9 +391,16 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
                 else:
                     space.manage_delLocalRoles(space.users_with_local_role('Owner'))
                     self.setGroupOwnership(self.getGroupById(id), space)
+
+                # Hook to allow doing other things after grouparea creation.
+                notify_script = getattr(workspaces, 'notifyGroupAreaCreated', None)
+                if notify_script is not None:
+                    notify_script()
+
+                # Re-indexation
                 portal_catalog = getToolByName( self, 'portal_catalog' )
                 portal_catalog.reindexObject(space)
-
+ 
     security.declareProtected(ManagePortal, 'getGroupWorkspaceType')
     def getGroupWorkspaceType(self):
         """Return the Type (as in TypesTool) to make the GroupWorkspace."""
@@ -412,26 +470,13 @@ class GroupsTool (UniqueObject, SimpleItem, ActionProviderBase):
         parent = self.aq_inner.aq_parent
         base = getattr(parent, 'aq_base', None)
         if hasattr(base, 'portal_groupdata'):
-##             # Apply any role mapping if we have it
-##             if hasattr(self, 'role_map'):
-##                 for portal_role in self.role_map.keys():
-##                     if (self.role_map.get(portal_role) in u.roles and
-##                             portal_role not in u.roles):
-##                         u.roles.append(portal_role)
-
             # Get portal_groupdata to do the wrapping.
+            Log(LOG_DEBUG, "parent", parent)
             gd = getToolByName(parent, 'portal_groupdata')
+            Log(LOG_DEBUG, "group data", gd)
             try:
                 #log("wrapping group %s" % g)
                 portal_group = gd.wrapGroup(g)
-
-                # Check for the member area creation flag and
-                # take appropriate (non-) action
-
-                # DISABLED: do it upon group creation
-                #if self.getGroupWorkspacesCreationFlag():
-                #    self.createGrouparea(portal_group.getGroupName())
-
                 return portal_group
 
             except:
