@@ -1,61 +1,41 @@
-##############################################################################
-#    Copyright (C) 2001, 2002, 2003 Lalo Martins <lalo@laranja.org>,
-#                  Zope Corporation and Contributors
-
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
-
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 """Placeless Translation Service for providing I18n to file-based code.
 
-$Id: PlacelessTranslationService.py,v 1.37.2.1 2004/11/25 22:23:35 longsleep Exp $
+$Id: PlacelessTranslationService.py 29774 2006-09-02 21:06:59Z alecm $
 """
 
 import sys, os, re, fnmatch
-from types import DictType, StringType, UnicodeType
+from types import StringType, UnicodeType
 
 import Globals
+from ExtensionClass import Base
+from Acquisition import ImplicitAcquisitionWrapper
 from AccessControl import ClassSecurityInfo
 from AccessControl.Permissions import view, view_management_screens
 from Globals import InitializeClass
 from OFS.Folder import Folder
 from ZPublisher.HTTPRequest import HTTPRequest
 
-from GettextMessageCatalog import BrokenMessageCatalog, GettextMessageCatalog, translationRegistry, getMessage
+from GettextMessageCatalog import BrokenMessageCatalog
+from GettextMessageCatalog import GettextMessageCatalog
+from GettextMessageCatalog import translationRegistry
+from GettextMessageCatalog import rtlRegistry
+from GettextMessageCatalog import getMessage
 from Negotiator import negotiator
 from Domain import Domain
-from utils import log, Registry, INFO, BLATHER, PROBLEM
+import logging
+from utils import log, Registry
 from msgfmt import PoSyntaxError
 
-try:
-    True
-except NameError:
-    True=1
-    False=0
+from Tracker import global_tracker
+from GettextMessageCatalog import ptFile
 
-try:
-    from pax import XML
-except:
-    def XML(v):
-        return str(v)
+PTS_IS_RTL = '_pts_is_rtl'
 
 _marker = []
 
-def map_get(map, name):
-    return map.get(name)
-
 # Setting up some regular expressions for finding interpolation variables in
 # the text.
-NAME_RE = r"[a-zA-Z][a-zA-Z0-9_]*"
+NAME_RE = r"[a-zA-Z][-a-zA-Z0-9_]*"
 _interp_regex = re.compile(r'(?<!\$)(\$(?:%(n)s|{%(n)s}))' %({'n': NAME_RE}))
 _get_var_regex = re.compile(r'%(n)s' %({'n': NAME_RE}))
 
@@ -76,16 +56,12 @@ fbcatalogRegistry = Registry()
 registerFBCatalog = fbcatalogRegistry.register
 
 
-class PTSWrapper:
+class PTSWrapper(Base):
     """
     Wrap the persistent PTS since persistent
     objects cant be passed around threads
     """
 
-    #XXX: better have a real seperation between
-    #     control panel and translation service
-    #     to avoid these zodb traversals
-
     security = ClassSecurityInfo()
 
     def __init__(self, service):
@@ -108,11 +84,11 @@ class PTSWrapper:
         get the real service and call its translate method
         return default if service couldnt be retrieved
         """
-        
-        # this is useful for GettextMessageCatalog
-        # see the end of GettextMessageCatalog.py for details        
+
+        # this is useful for GettextMessageCatalog see the end of
+        # GettextMessageCatalog.py for details
         __pts_caller_backcount__ = 1
-        
+
         service = self.load(context)
         if not service: return default
         return service.translate(domain, msgid, mapping, context, target_language, default)
@@ -121,19 +97,18 @@ class PTSWrapper:
     def utranslate(self, domain, msgid, mapping=None, context=None,
                   target_language=None, default=None):
         """
-        translate a message using unicode
-        see translate()
+        Translate a message using unicode. See translate().
         """
         service = self.load(context)
         if not service: return default
         return service.utranslate(domain, msgid, mapping, context, target_language, default)
 
-    security.declarePublic(view, 'getLanguageName')
+    security.declareProtected(view, 'getLanguageName')
     def getLanguageName(self, code, context):
         service = self.load(context)
         return service.getLanguageName(code)
 
-    security.declarePublic(view, 'getLanguages')
+    security.declareProtected(view, 'getLanguages')
     def getLanguages(self, context, domain=None):
         service = self.load(context)
         return service.getLanguages(domain)
@@ -143,9 +118,16 @@ class PTSWrapper:
         service = self.load(context)
         return service.negotiate_language(context.REQUEST,domain)
 
+    security.declarePublic('isRTL')
+    def isRTL(self, context, domain):
+        service = self.load(context)
+        # Default to LtR
+        if service is None: return False
+        return service.isRTL(context.REQUEST,domain)
+
     def __repr__(self):
         """
-        return a string representation
+        Return a string representation
         """
         return "<PTSWrapper for %s>" % '/'.join(self._path)
 
@@ -159,13 +141,20 @@ class PlacelessTranslationService(Folder):
     meta_type = title = 'Placeless Translation Service'
     icon = 'misc_/PlacelessTranslationService/PlacelessTranslationService.png'
     # major, minor, patchlevel, internal
-    # internal is always 0 on releases; if you hack this internally, increment it
+    # internal is always 0 on releases
+    # if you hack this internally, increment it
     # -3 for alpha, -2 for beta, -1 for release candidate
-    # for forked releases internal is always 99
-    # use an internal of >99 to recreate the PTS at every startup (development mode)
-    _class_version = (1, 1, 2, 0)
+    # use an internal of > 99 to recreate the PTS at every startup
+    # (development mode)
+    _class_version = (1, 3, 2, 0)
     all_meta_types = ()
 
+    manage_options = Folder.manage_options + (
+        {'label': 'Tracker', 'action': 'manage_trackerForm'},)
+
+    manage_trackerForm = ptFile(
+        'manage_trackerForm', globals(), 'www', 'manage_trackerForm')
+
     security = ClassSecurityInfo()
 
     def __init__(self, default_domain='global', fallbacks=None):
@@ -183,8 +172,6 @@ class PlacelessTranslationService(Folder):
         self._fallbacks = fallbacks
 
     def _registerMessageCatalog(self, catalog):
-
-
         # dont register broken message catalogs
         if isinstance(catalog, BrokenMessageCatalog): return
 
@@ -200,7 +187,8 @@ class PlacelessTranslationService(Folder):
                 combo.remove(catalog.getIdentifier())
             except ValueError:
                 continue
-            if not combo: # removed the last catalog for a language/domain combination
+            if not combo: # removed the last catalog for a
+                          # language/domain combination
                 del clist[key]
 
     def _unregisterMessageCatalog(self, catalog):
@@ -209,7 +197,7 @@ class PlacelessTranslationService(Folder):
         self._p_changed = 1
 
     security.declarePrivate('calculatePoId')
-    def calculatePoId(self, name, popath):
+    def calculatePoId(self, name, popath, language=None, domain=None):
         """Calulate the po id
         """
         # instance, software and global catalog path for i18n and locales
@@ -233,8 +221,8 @@ class PlacelessTranslationService(Folder):
             path = popath[len(gcLocPath):]
             isGlobalCatalog = True
         else:
-            # po file is located at a strange place
-            # calculate the name using the position of the i18n/locales directory
+            # po file is located at a strange place calculate the name using
+            # the position of the i18n/locales directory
             p = popath.split(os.sep)
             try:
                 idx = p.index('i18n')
@@ -253,18 +241,22 @@ class PlacelessTranslationService(Folder):
         else:
             pre = 'GlobalCatalogs'
 
-        return '%s-%s' % (pre, name)
+        if language and domain:
+            return "%s-%s-%s.po" % (pre, language, domain)
+        else:
+            return '%s-%s' % (pre, name)
 
     def _load_catalog_file(self, name, popath, language=None, domain=None):
         """
         create catalog instances in ZODB
         """
-
-        id = self.calculatePoId(name, popath)
+        id = self.calculatePoId(name, popath, language=language, domain=domain)
 
         # validate id
-        try: self._checkId(id, 1)
-        except: id=name # fallback mode for borked paths
+        try:
+            self._checkId(id, 1)
+        except:
+            id=name # fallback mode for borked paths
 
         # the po file path
         pofile = os.path.join(popath, name)
@@ -275,23 +267,25 @@ class PlacelessTranslationService(Folder):
                 # remove broken catalog
                 self._delObject(id)
                 ob = _marker
-        except: pass
+        except:
+            pass
         try:
             if ob is _marker:
                 self.addCatalog(GettextMessageCatalog(id, pofile, language, domain))
             else:
                 self.reloadCatalog(ob)
         except IOError:
-            # io error probably cause of missing or
-            # not accessable
+            # io error probably cause of missing or not accessable
             try:
                 # remove false catalog from PTS instance
                 self._delObject(id)
             except:
                 pass
+        except KeyboardInterrupt:
+            raise
         except:
             exc=sys.exc_info()
-            log('Message Catalog has errors', PROBLEM, name, exc)
+            log('Message Catalog has errors', logging.WARNING, pofile, exc)
             self.addCatalog(BrokenMessageCatalog(id, pofile, exc))
 
     def _load_i18n_dir(self, basepath):
@@ -301,21 +295,25 @@ class PlacelessTranslationService(Folder):
             Products/MyProduct/i18n/*.po
         The language and domain are stored in the po file
         """
-        log('looking into ' + basepath, BLATHER)
+        log('looking into ' + basepath, logging.DEBUG)
         if not os.path.isdir(basepath):
-            log('it does not exist', BLATHER)
+            log('it does not exist', logging.DEBUG)
             return
 
         # print deprecation warning for mo files
         depr_names = fnmatch.filter(os.listdir(basepath), '*.mo')
         if depr_names:
             import warnings
-            warnings.warn('Compiled po files (*.mo) found in %s. PlacelessTranslationService now compiles mo files automatically. All mo files have been ignored.' % basepath, DeprecationWarning, stacklevel=4)
+            warnings.warn(
+                'Compiled po files (*.mo) found in %s. '
+                'PlacelessTranslationService now compiles '
+                'mo files automatically. All mo files have '
+                'been ignored.' % basepath, DeprecationWarning, stacklevel=4)
 
         # load po files
         names = fnmatch.filter(os.listdir(basepath), '*.po')
         if not names:
-            log('nothing found', BLATHER)
+            log('nothing found', logging.DEBUG)
             return
         for name in names:
             self._load_catalog_file(name, basepath)
@@ -331,9 +329,9 @@ class PlacelessTranslationService(Folder):
         file (e.g. locales/de/LC_MESSAGES/plone.po)
         """
         found=[]
-        log('looking into ' + basepath, BLATHER)
+        log('looking into ' + basepath, logging.DEBUG)
         if not os.path.isdir(basepath):
-            log('it does not exist', BLATHER)
+            log('it does not exist', logging.DEBUG)
             return
 
         for lang in os.listdir(basepath):
@@ -352,7 +350,7 @@ class PlacelessTranslationService(Folder):
                 self._load_catalog_file(name, msgpath, lang, domain)
 
         if not found:
-            log('nothing found', BLATHER)
+            log('nothing found', logging.DEBUG)
             return
         log('Initialized:', detail = repr(found) + (' from %s\n' % basepath))
 
@@ -361,10 +359,10 @@ class PlacelessTranslationService(Folder):
         context = getattr(context, 'REQUEST', context)
         if not isinstance(context, HTTPRequest):
             # try to recover
-            log('Using get_request patch.', severity=INFO)
+            log('Using get_request patch.', severity=logging.INFO)
             # XXX: import the get_request method
-            # this will fail the first time we need it if the patch wasn't applied
-            # before
+            # this will fail the first time we need it if the patch
+            # wasn't applied before
             try:
                 from Globals import get_request
             except ImportError:
@@ -415,16 +413,16 @@ class PlacelessTranslationService(Folder):
         # ZPT passes the object as context.  That's wrong according to spec.
         context = self._getContext(context)
 
+        if target_language is None:
+            target_language = self.negotiate_language(context, domain)
+
         # cache catalog names to speed up because this method is called
         # for every msgid
-        cache_name = 'PTS_catalog_names_%s_%s' % (domain, target_language or 'none')
+        cache_name = '_pts_catalog_names_%s_%s' % (domain, target_language or 'none')
         cached_catalog_names = context.get(cache_name, None)
         if cached_catalog_names:
             return [translationRegistry[name] for name in cached_catalog_names]
 
-        if target_language is None:
-            target_language = self.negotiate_language(context, domain)
-
         # get the catalogs for translations
         catalog_names = catalogRegistry.get((target_language, domain), ()) or \
                         fbcatalogRegistry.get((target_language, domain), ())
@@ -453,7 +451,15 @@ class PlacelessTranslationService(Folder):
         # in the request. It may cause reference circles and cause memory leaks
         context.set(cache_name, catalog_names)
 
-        return [translationRegistry[name] for name in catalog_names ]
+        # test for right to left language
+        if not context.has_key(PTS_IS_RTL):
+            context.set(PTS_IS_RTL, False)
+        for name in catalog_names:
+            if rtlRegistry.get(name):
+                context.set(PTS_IS_RTL, True)
+                break
+
+        return [translationRegistry[name] for name in catalog_names]
 
     security.declarePrivate('setLanguageFallbacks')
     def setLanguageFallbacks(self, fallbacks=None):
@@ -461,7 +467,6 @@ class PlacelessTranslationService(Folder):
             fallbacks = LANGUAGE_FALLBACKS
         self._fallbacks = fallbacks
 
-
     security.declareProtected(view, 'getLanguageName')
     def getLanguageName(self, code):
         for (ccode, cdomain), cnames in catalogRegistry.items():
@@ -489,6 +494,17 @@ class PlacelessTranslationService(Folder):
         l.sort()
         return l
 
+    security.declareProtected(view, 'isRTL')
+    def isRTL(self, context, domain):
+        """get RTL settings
+        """
+        context = self._getContext(context)
+        pts_is_rtl = context.get(PTS_IS_RTL, None)
+        if pts_is_rtl is None:
+            # call getCatalogsForTranslation to initialize the negotiator
+            self.getCatalogsForTranslation(context, domain)
+        return context.get(PTS_IS_RTL, False)
+
     security.declareProtected(view, 'utranslate')
     def utranslate(self, domain, msgid, mapping=None, context=None,
                   target_language=None, default=None):
@@ -504,12 +520,14 @@ class PlacelessTranslationService(Folder):
         """
         translate a message using the default encoding
         """
+        global global_tracker
         if not msgid:
             # refuse to translate an empty msgid
             return default
 
         # ZPT passes the object as context.  That's wrong according to spec.
         context = self._getContext(context)
+        text = msgid
 
         catalogs = self.getCatalogsForTranslation(context, domain, target_language)
         for catalog in catalogs:
@@ -519,22 +537,17 @@ class PlacelessTranslationService(Folder):
                 # it's not in this catalog, try the next one
                 continue
             # found! negotiate output encodings now
-            if hasattr(context, 'pt_output_encoding'):
-                # OpenPT
-                encodings = catalog._info.get('preferred-encodings', '').split()
-                if encodings:
-                    context.pt_output_encoding.restrict(catalog, encodings)
-            elif not as_unicode:
-                # ZPT probably
+            if not as_unicode:
                 # ask HTTPResponse to encode it for us
                 text = context.RESPONSE._encode_unicode(text)
             break
         else:
-            # Did the fallback fail?  Sigh, use the default.
-            # OpenTAL provides a default text.
-            # TAL doesn't but will use the default
-            # if None is returned
-            text = default
+            # Did the fallback fail? Sigh, use the default if it is not None.
+            if default is not None:
+                text = default
+            if target_language is None:
+                target_language = self.negotiate_language(context, domain)
+            global_tracker.recordFailure(domain, msgid, target_language)
 
         # Now we need to do the interpolation
         return self.interpolate(text, mapping)
@@ -562,53 +575,29 @@ class PlacelessTranslationService(Folder):
         """
         Insert the data passed from mapping into the text
         """
+        # If the mapping does not exist or is empty, make a
+        # "raw translation" without interpolation.
+        if not mapping:
+            return text
 
-        # XXX: why this big try/except?
-        try:
+        # Find all the spots we want to substitute
+        to_replace = _interp_regex.findall(text)
 
-            # If the mapping does not exist, make a "raw translation" without
-            # interpolation.
-            if mapping is None or type(text) not in (StringType, UnicodeType):
-                # silly wabbit!
-                return text
-
-            get = map_get
+        # Now substitute with the variables in mapping
+        for string in to_replace:
+            var = _get_var_regex.findall(string)[0]
+            value = mapping.get(var, None)
+            if value is None:
+                value = string
             try:
-                mapping.get('')
-            except AttributeError:
-                get = getattr
-
-            # Find all the spots we want to substitute
-            to_replace = _interp_regex.findall(text)
-
-            # ZPT (string) or OpenPT (unicode)?
-            if type(text) is StringType:
-                conv = str
-            else:
-                conv = XML
-
-            # Now substitute with the variables in mapping
-            for string in to_replace:
-                var = _get_var_regex.findall(string)[0]
-                value = get(mapping, var)
-                try:
-                    value = value()
-                except (TypeError, AttributeError):
-                    pass
-                if value is None:
-                    value = string
-                if type(value) not in (StringType, UnicodeType):
-                    # FIXME: we shouldn't do this. We should instead
-                    # return a list. But i'm not sure about how to use
-                    # the regex to split the text.
-                    value = conv(value)
+                if not isinstance(value, basestring):
+                    value = str(value)
+                if isinstance(text, unicode):
+                    value = u'%s' % value
                 text = text.replace(string, value)
-
-            return text
-
-        except:
-            log('interpolation error', PROBLEM, error=sys.exc_info())
-            return text
+            except UnicodeDecodeError, msg:
+                log('Decoding problem in: %s %s' % (text, msg), logging.WARNING)
+        return text
 
     security.declareProtected(view_management_screens, 'manage_main')
     def manage_main(self, REQUEST, *a, **kw):
@@ -619,11 +608,30 @@ class PlacelessTranslationService(Folder):
         if REQUEST is self and a:
             REQUEST = a[0]
             a = a[1:]
-        r = Folder.manage_main(self, self, REQUEST, *a, **kw)
+        # wrap the special dtml method Folder.manage_main into a valid
+        # acquisition context. Required for Zope 2.8+.
+        try:
+            r = Folder.manage_main(self, self, REQUEST, *a, **kw)
+        except AttributeError:
+            manage_main = ImplicitAcquisitionWrapper(Folder.manage_main, self)
+            r = manage_main(self, self, REQUEST, *a, **kw)
         if type(r) is UnicodeType:
             r = r.encode('utf-8')
         REQUEST.RESPONSE.setHeader('Content-type', 'text/html; charset=utf-8')
         return r
 
+    security.declareProtected(view_management_screens, 'manage_tracker')
+    def manage_tracker(self, REQUEST=None):
+        """Defers all to the global_tracker
+        """
+        global global_tracker
+        return global_tracker.manage_tracker(self, REQUEST)
+
+    security.declareProtected(view, 'getTracker')
+    def getTracker(self):
+        """The domain tracker of that PTS (used only for ZMI template)
+        """
+        global global_tracker
+        return global_tracker
 
 InitializeClass(PlacelessTranslationService)
