@@ -1,45 +1,29 @@
-##############################################################################
-#    Copyright (C) 2001, 2002, 2003 Lalo Martins <lalo@laranja.org>,
-#                  and Contributors
+"""
+$Id: __init__.py 21856 2006-04-05 23:42:31Z hannosch $
+"""
 
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
-#    (at your option) any later version.
-
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-
-#    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
-__version__ = '''
-$Id: __init__.py,v 1.16 2004/06/23 12:09:32 tiran Exp $
-'''.strip()
-
-import os, sys
-import fnmatch
+import os
 
 from Products.PageTemplates.GlobalTranslationService import \
     setGlobalTranslationService, getGlobalTranslationService
 import Globals
+pts_globals = globals()
+
 from OFS.Application import get_products
 from AccessControl import ModuleSecurityInfo, allow_module
 from AccessControl.Permissions import view
 
 import PatchStringIO # patch at first
-from PlacelessTranslationService import PlacelessTranslationService, PTSWrapper
-from utils import log, WARNING, BLATHER, PROBLEM
+from PlacelessTranslationService import PlacelessTranslationService
+from PlacelessTranslationService import PTSWrapper
+from PlacelessTranslationService import PTS_IS_RTL
+
+import logging
+from utils import log
+
 from Negotiator import negotiator, setCookieLanguage
 import TranslateTags
-
-# in python 2.1 fnmatch doesn't have the filter function
-if not hasattr(fnmatch, 'filter'):
-    def fnfilter(names, pattern):
-        return [name for name in names if fnmatch.fnmatch(name, pattern)]
-    fnmatch.filter = fnfilter
+from GettextMessageCatalog import purgeMoFileCache
 
 # this is for packages that need to initialize stuff after we're done
 notify_initialized = []
@@ -61,33 +45,43 @@ misc_ = {
 # set product-wide attrs for importing
 security = ModuleSecurityInfo('Products.PlacelessTranslationService')
 allow_module('Products.PlacelessTranslationService')
-allow_module('Products.PlacelessTranslationService.MessageID')
 
 security.declareProtected(view, 'getTranslationService')
 def getTranslationService():
-    """ returns the PTS instance """
+    """returns the PTS instance
+    """
     return translation_service
 
 security.declareProtected(view, 'translate')
 def translate(*args, **kwargs):
-    """ see PlaceslessTranslationService.PlaceslessTranslationService """
+    """see PlaceslessTranslationService.PlaceslessTranslationService
+    """
     return getTranslationService().translate(*args, **kwargs)
 
 security.declareProtected(view, 'utranslate')
 def utranslate(*args, **kwargs):
-    """ see PlaceslessTranslationService.PlaceslessTranslationService """
+    """see PlaceslessTranslationService.PlaceslessTranslationService
+    """
     return getTranslationService().utranslate(*args, **kwargs)
 
 security.declareProtected(view, 'getLanguages')
 def getLanguages(*args, **kwargs):
-    """ see PlaceslessTranslationService.PlaceslessTranslationService """
+    """see PlaceslessTranslationService.PlaceslessTranslationService
+    """
     return getTranslationService().getLanguages(*args, **kwargs)
 
 security.declareProtected(view, 'getLanguageName')
 def getLanguageName(*args, **kwargs):
-    """ see PlaceslessTranslationService.PTSWrapper """
+    """see PlaceslessTranslationService.PTSWrapper
+    """
     return getTranslationService().getLanguageName(*args, **kwargs)
 
+security.declareProtected(view, 'isRTL')
+def isRTL(context, domain):
+    """Returns true for a rtl language and false for a ltr language
+    """
+    return getTranslationService().isRTL(context, domain)
+
 security.declareProtected(view, 'setCookieLanguage')
 # imported from the Negotiator
 
@@ -96,7 +90,7 @@ security.declareProtected(view, 'negotiate')
 def negotiate(langs, context):
     """ deprecated! """
     if not negotiateDeprecatedLogged:
-        log('Products.PlacelessTranslationService.negotiate() is deprecated', WARNING)
+        log('Products.PlacelessTranslationService.negotiate() is deprecated', logging.WARNING)
         negotiateDeprecatedLogged = 1
     return negotiator.negotiate(langs, context, 'language')
 
@@ -107,21 +101,29 @@ def make_translation_service(cp):
     translation_service = PlacelessTranslationService('default')
     translation_service.id = cp_id
     cp._setObject(cp_id, translation_service)
+    translation_service = PTSWrapper(translation_service)
     return getattr(cp, cp_id)
 
 def initialize(context):
     # hook into the Control Panel
     global translation_service
+
+    # allow for disabling PTS entirely by setting a environment variable.
+    if bool(os.getenv('DISABLE_PTS')):
+        log('Disabled by environment variable "DISABLE_PTS".', logging.WARNING)
+        return
+
     cp = context._ProductContext__app.Control_Panel # argh
     if cp_id in cp.objectIds():
         cp_ts = getattr(cp, cp_id)
         # use the ts in the acquisition context of the control panel
         # translation_service = translation_service.__of__(cp)
-        translation_service = cp_ts
+        translation_service = PTSWrapper(cp_ts)
     else:
         cp_ts = make_translation_service(cp)
 
-    # don't touch - this is the last version that didn't have the attribute (0.4)
+    # don't touch - this is the last version that didn't have the
+    # attribute (0.4)
     instance_version = getattr(cp_ts, '_instance_version', (0, 4, 0, 0))
     if instance_version[3] > 99:
         log('development mode: translation service recreated',
@@ -133,12 +135,11 @@ def initialize(context):
         log('outdated translation service found, recreating',
             detail = '(found %s.%s.%s.%s)\n' % instance_version)
         cp._delObject(cp_id)
+        purgeMoFileCache()
         cp_ts = make_translation_service(cp)
 
-
     # sweep products
-    log('products: %r' % get_products(), BLATHER)
-    ploneDir = None
+    log('products: %r' % get_products(), logging.DEBUG)
     for prod in get_products():
         # prod is a tuple in the form:
         # (priority, dir_name, index, base_dir) for each Product directory
@@ -156,11 +157,12 @@ def initialize(context):
 
     # didn't found any catalogs
     if not cp_ts.objectIds():
-        log('no translations found!', PROBLEM)
+        log('no translations found!', logging.DEBUG)
 
     # set ZPT's translation service
-    # NOTE: since this registry is a global var we cant register the persistent service itself (zodb connection)
-    #       therefor a wrapper is created around it
+    # NOTE: since this registry is a global var we cant register the
+    #       persistent service itself (zodb connection) therefore a
+    #       wrapper is created around it
     setGlobalTranslationService(PTSWrapper(cp_ts))
 
     # notify anyone who needs it
